# -*- coding: utf-8 -*-
"""Utils to translate FTIs / zope.schema interfaces to JSON schemas.

The basic idea here is to instantiate a minimal z3c form, and then have
plone.autoform work its magic on it to process all the fields, and apply
any p.autoform directives (fieldsets, field modes, omitted fields, field
permissions, widgets).

Also schema interface inheritance (IRO) and additional schemata from behaviors
are factored into how the final resulting fieldsets are composed.

This approach should ensure that all these directives get respected and
processed the same way they would for a server-rendered form.
"""

from collections import OrderedDict
from copy import copy
from plone.autoform.form import AutoExtensibleForm
from plone.autoform.interfaces import IParameterizedWidget
from plone.autoform.interfaces import WIDGETS_KEY
from plone.dexterity.utils import getAdditionalSchemata
from plone.restapi.serializer.converters import IJsonCompatible
from plone.restapi.types.interfaces import IJsonSchemaProvider
from plone.supermodel.utils import mergedTaggedValueDict
from Products.CMFCore.utils import getToolByName
from z3c.form import form as z3c_form
from zope.component import getMultiAdapter
from zope.component import queryMultiAdapter
from zope.component.hooks import getSite
from zope.globalrequest import getRequest
from zope.i18n import translate


def create_form(context, request, base_schema, additional_schemata=None):
    """Create a minimal, standalone z3c form and run the field processing
    logic of plone.autoform on it.
    """
    if additional_schemata is None:
        additional_schemata = ()

    class SchemaForm(AutoExtensibleForm, z3c_form.AddForm):
        schema = base_schema
        additionalSchemata = additional_schemata
        ignoreContext = True

    form = SchemaForm(context, request)
    form.updateFieldsFromSchemata()
    return form


def iter_fields(fieldsets):
    """Iterate over a flat list of fields, given a list of fieldset dicts
    as returned by `get_fieldsets`.
    """
    for fieldset in fieldsets:
        for field in fieldset["fields"]:
            yield field


def get_form_fieldsets(form):
    """ Get fieldsets from form
    """
    fieldsets = []
    form_fields = getattr(form, "fields", {})
    fields_values = list(form_fields.values())
    if form_fields:
        fieldsets.append(
            {
                "id": "default",
                "title": translate(
                    "label_schema_default",
                    default="Default",
                    domain="plone",
                    context=getRequest(),
                ),
                "fields": fields_values,
            }
        )

    # Additional fieldsets (AKA z3c.form groups)
    for group in getattr(form, "groups", []):
        fieldset = {
            "id": group.__name__,
            "title": translate(group.label, context=getRequest()),
            "fields": list(group.fields.values()),
        }
        fieldsets.append(fieldset)
    return fieldsets


def get_fieldsets(context, request, schema, additional_schemata=None):
    """Given a base schema, and optionally some additional schemata,
    build a list of fieldsets with the corresponding z3c.form fields in them.
    """
    form = create_form(context, request, schema, additional_schemata)
    return get_form_fieldsets(form)


def get_fieldset_infos(fieldsets):
    """Given a list of fieldset dicts as returned by `get_fieldsets()`,
    return a list of fieldset info dicts that contain the (short) field name
    instead of the actual field instance.
    """
    fieldset_infos = []
    for fieldset in fieldsets:
        fs_info = copy(fieldset)
        fs_info["fields"] = [f.field.getName() for f in fs_info["fields"]]
        fieldset_infos.append(fs_info)
    return fieldset_infos


def get_jsonschema_properties(
    context, request, fieldsets, prefix="", excluded_fields=None
):
    """Build a JSON schema 'properties' list, based on a list of fieldset
    dicts as returned by `get_fieldsets()`.
    """
    properties = OrderedDict()
    if excluded_fields is None:
        excluded_fields = []

    for field in iter_fields(fieldsets):
        fieldname = field.field.getName()
        if fieldname not in excluded_fields:
            # We need to special case relatedItems not to render choices
            # so we try a named adapter first and fallback to unnamed ones.
            adapter = queryMultiAdapter(
                (field.field, context, request),
                interface=IJsonSchemaProvider,
                name=field.__name__,
            )

            adapter = adapter or getMultiAdapter(
                (field.field, context, request), interface=IJsonSchemaProvider
            )

            adapter.prefix = prefix
            if prefix:
                fieldname = ".".join([prefix, fieldname])

            properties[fieldname] = adapter.get_schema()

    return properties


def get_widget_params(schemas):
    params = {}
    for schema in schemas:
        if not schema:
            continue
        tagged_values = mergedTaggedValueDict(schema, WIDGETS_KEY)
        for field_name in schema:
            widget = tagged_values.get(field_name)
            if IParameterizedWidget.providedBy(widget) and widget.params:
                params[field_name] = {}
                for k, v in widget.params.items():
                    if callable(v):
                        v = v()
                    params[field_name][k] = v
    return params


def get_jsonschema_for_fti(fti, context, request, excluded_fields=None):
    """Build a complete JSON schema for the given FTI.
    """
    if excluded_fields is None:
        excluded_fields = []

    # We try..except lookupSchema here, so we still get FTI information
    # through /@types/{typeid} for non-DX type, notably the "Plone Site" type.
    try:
        schema = fti.lookupSchema()
    except AttributeError:
        schema = None
        fieldsets = ()
        additional_schemata = ()
    else:
        additional_schemata = tuple(getAdditionalSchemata(portal_type=fti.id))
        fieldsets = get_fieldsets(context, request, schema, additional_schemata)

    # Build JSON schema properties
    properties = get_jsonschema_properties(
        context, request, fieldsets, excluded_fields=excluded_fields
    )

    # Determine required fields
    required = []
    for field in iter_fields(fieldsets):
        if field.field.required:
            required.append(field.field.getName())

    # Include field modes
    for field in iter_fields(fieldsets):
        if field.mode:
            properties[field.field.getName()]["mode"] = field.mode

    return {
        "type": "object",
        "title": translate(fti.Title(), context=getRequest()),
        "properties": IJsonCompatible(properties),
        "required": required,
        "fieldsets": get_fieldset_infos(fieldsets),
        "layouts": getattr(fti, "view_methods", []),
    }


def get_jsonschema_for_portal_type(portal_type, context, request, excluded_fields=None):
    """Build a complete JSON schema for the given portal_type.
    """
    ttool = getToolByName(context, "portal_types")
    fti = ttool[portal_type]
    return get_jsonschema_for_fti(
        fti, context, request, excluded_fields=excluded_fields
    )


def get_vocab_like_url(endpoint, locator, context, request):
    try:
        context_url = context.absolute_url()
    except AttributeError:
        portal = getSite()
        context_url = portal.absolute_url()
    return "/".join((context_url, endpoint, locator))


def get_vocabulary_url(vocab_name, context, request):
    return get_vocab_like_url("@vocabularies", vocab_name, context, request)


def get_querysource_url(field, context, request):
    return get_vocab_like_url("@querysources", field.getName(), context, request)


def get_source_url(field, context, request):
    return get_vocab_like_url("@sources", field.getName(), context, request)



# -*- coding: utf-8 -*-
from plone.restapi.batching import HypermediaBatch
from plone.restapi.interfaces import ISerializeToJson
from plone.restapi.interfaces import ISerializeToJsonSummary
from plone.restapi.serializer.expansion import expandable_elements
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.interfaces import IPloneSiteRoot
from zope.component import adapter
from zope.component import getMultiAdapter
from zope.interface import implementer
from zope.interface import Interface

import json


@implementer(ISerializeToJson)
@adapter(IPloneSiteRoot, Interface)
class SerializeSiteRootToJson(object):
    def __init__(self, context, request):
        self.context = context
        self.request = request

    def _build_query(self):
        path = "/".join(self.context.getPhysicalPath())
        query = {
            "path": {"depth": 1, "query": path},
            "sort_on": "getObjPositionInParent",
        }
        return query

    def __call__(self, version=None):
        version = "current" if version is None else version
        if version != "current":
            return {}

        query = self._build_query()

        catalog = getToolByName(self.context, "portal_catalog")
        brains = catalog(query)

        batch = HypermediaBatch(self.request, brains)

        result = {
            # '@context': 'http://www.w3.org/ns/hydra/context.jsonld',
            "@id": batch.canonical_url,
            "id": self.context.id,
            "@type": "Plone Site",
            "title": self.context.Title(),
            "parent": {},
            "is_folderish": True,
            "description": self.context.description,
            "blocks": json.loads(getattr(self.context, "blocks", "{}")),
            "blocks_layout": json.loads(
                getattr(self.context, "blocks_layout", "{}")
            ),  # noqa
        }

        # Insert expandable elements
        result.update(expandable_elements(self.context, self.request))

        result["items_total"] = batch.items_total
        if batch.links:
            result["batching"] = batch.links

        result["items"] = [
            getMultiAdapter((brain, self.request), ISerializeToJsonSummary)()
            for brain in batch
        ]

        return result



# -*- coding: utf-8 -*-
from AccessControl import allow_module
from AccessControl.Permissions import add_user_folders
from plone.restapi.pas import plugin
from Products.PluggableAuthService.PluggableAuthService import registerMultiPlugin
from zope.i18nmessageid import MessageFactory

import pkg_resources


_ = MessageFactory("plone.restapi")
PROJECT_NAME = "plone.restapi"


allow_module("json")

try:
    pkg_resources.get_distribution("plone.app.testing")
    REGISTER_TEST_TYPES = True
except pkg_resources.DistributionNotFound:  # pragma: no cover
    REGISTER_TEST_TYPES = False

try:
    pkg_resources.get_distribution("plone.app.contenttypes")
    HAS_PLONE_APP_CONTENTTYPES = True
except pkg_resources.DistributionNotFound:  # pragma: no cover
    HAS_PLONE_APP_CONTENTTYPES = False

try:
    pkg_resources.get_distribution("Products.Archetypes")
except pkg_resources.DistributionNotFound:
    HAS_AT = False
else:
    HAS_AT = True


def initialize(context):
    registerMultiPlugin(plugin.JWTAuthenticationPlugin.meta_type)
    context.registerClass(
        plugin.JWTAuthenticationPlugin,
        permission=add_user_folders,
        constructors=(
            plugin.manage_addJWTAuthenticationPlugin,
            plugin.addJWTAuthenticationPlugin,
        ),
        visibility=None,
    )

    if HAS_AT and REGISTER_TEST_TYPES:
        from Products.Archetypes.ArchetypeTool import process_types, listTypes
        from Products.CMFCore import permissions
        from Products.CMFCore import utils
        from plone.restapi.tests.attypes import PROJECTNAME

        content_types, constructors, ftis = process_types(
            listTypes(PROJECTNAME), PROJECTNAME
        )

        utils.ContentInit(
            "%s Content" % PROJECTNAME,
            content_types=content_types,
            permission=permissions.AddPortalContent,
            extra_constructors=constructors,
            fti=ftis,
        ).initialize(context)



# -*- coding: utf-8 -*-
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.dexterity.interfaces import IDexterityFTI
from plone.dexterity.utils import createContentInContainer
from plone.restapi.testing import PLONE_RESTAPI_DX_FUNCTIONAL_TESTING
from plone.restapi.testing import RelativeSession
from zope.component import queryUtility

import transaction
import unittest


class TestSearchTextInBlocks(unittest.TestCase):

    layer = PLONE_RESTAPI_DX_FUNCTIONAL_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)

        fti = queryUtility(IDexterityFTI, name="Document")
        behavior_list = [a for a in fti.behaviors]
        behavior_list.append("volto.blocks")
        fti.behaviors = tuple(behavior_list)

        self.doc = createContentInContainer(
            self.portal, u"Document", id=u"doc", title=u"A document"
        )
        transaction.commit()

    def tearDown(self):
        self.api_session.close()

    def test_search_text(self):
        response = self.api_session.patch(
            "/doc",
            json={
                "blocks": {
                    "uuid1": {
                        "@type": "text",
                        "text": {
                            "blocks": [
                                {
                                    "data": {},
                                    "depth": 0,
                                    "entityRanges": [],
                                    "inlineStyleRanges": [],
                                    "key": "acv4f",
                                    "text": "Plone " "text " "for " "block ",
                                    "type": "unstyled",
                                }
                            ],
                            "entityMap": {},
                        },
                    },
                    "uuid2": {
                        "@type": "text",
                        "text": {
                            "blocks": [
                                {
                                    "data": {},
                                    "depth": 0,
                                    "entityRanges": [],
                                    "inlineStyleRanges": [],
                                    "key": "1m9qt",
                                    "text": "Volto " "text " "for " "block ",
                                    "type": "unstyled",
                                }
                            ],
                            "entityMap": {},
                        },
                    },
                }
            },
        )

        self.assertEqual(response.status_code, 204)

        query = {"SearchableText": "Volto", "metadata_fields": "Title"}
        response = self.api_session.get("/@search", params=query)
        json_response = response.json()
        self.assertEqual(json_response["items_total"], 1)
        self.assertEqual(json_response["items"][0]["Title"], "A document")

        query = {"SearchableText": "Plone", "metadata_fields": "Title"}
        response = self.api_session.get("/@search", params=query)
        json_response = response.json()
        self.assertEqual(json_response["items_total"], 1)
        self.assertEqual(json_response["items"][0]["Title"], "A document")



# -*- coding: utf-8 -*-
from Acquisition import aq_inner
from Acquisition import aq_parent
from plone.restapi.batching import HypermediaBatch
from plone.restapi.deserializer import boolean_value
from plone.restapi.interfaces import IFieldSerializer
from plone.restapi.interfaces import ISerializeToJson
from plone.restapi.interfaces import ISerializeToJsonSummary
from plone.restapi.serializer.expansion import expandable_elements
from plone.restapi.serializer.nextprev import NextPrevious
from Products.Archetypes.interfaces import IBaseFolder
from Products.Archetypes.interfaces import IBaseObject
from Products.CMFCore.utils import getToolByName
from zope.component import adapter
from zope.component import getMultiAdapter
from zope.component import queryMultiAdapter
from zope.interface import implementer
from zope.interface import Interface


@implementer(ISerializeToJson)
@adapter(IBaseObject, Interface)
class SerializeToJson(object):
    def __init__(self, context, request):
        self.context = context
        self.request = request

    def getVersion(self, version):
        if version == "current":
            return self.context
        else:
            repo_tool = getToolByName(self.context, "portal_repository")
            return repo_tool.retrieve(self.context, int(version)).object

    def __call__(self, version=None, include_items=False):
        version = "current" if version is None else version

        obj = self.getVersion(version)
        parent = aq_parent(aq_inner(obj))
        parent_summary = getMultiAdapter(
            (parent, self.request), ISerializeToJsonSummary
        )()
        result = {
            # '@context': 'http://www.w3.org/ns/hydra/context.jsonld',
            "@id": obj.absolute_url(),
            "id": obj.id,
            "@type": obj.portal_type,
            "parent": parent_summary,
            "review_state": self._get_workflow_state(obj),
            "UID": obj.UID(),
            "layout": self.context.getLayout(),
            "is_folderish": False,
        }

        # Insert next/prev information
        nextprevious = NextPrevious(obj)
        result.update(
            {"previous_item": nextprevious.previous, "next_item": nextprevious.next}
        )

        # Insert expandable elements
        result.update(expandable_elements(self.context, self.request))

        for field in obj.Schema().fields():

            if "r" not in field.mode or not field.checkPermission(
                "r", obj
            ):  # noqa: E501
                continue

            name = field.getName()

            serializer = queryMultiAdapter(
                (field, self.context, self.request), IFieldSerializer
            )
            if serializer is not None:
                result[name] = serializer()

        return result

    def _get_workflow_state(self, obj):
        wftool = getToolByName(self.context, "portal_workflow")
        review_state = wftool.getInfoFor(ob=obj, name="review_state", default=None)
        return review_state


@implementer(ISerializeToJson)
@adapter(IBaseFolder, Interface)
class SerializeFolderToJson(SerializeToJson):
    def _build_query(self):
        path = "/".join(self.context.getPhysicalPath())
        query = {
            "path": {"depth": 1, "query": path},
            "sort_on": "getObjPositionInParent",
        }
        return query

    def __call__(self, version=None, include_items=True):
        folder_metadata = super(SerializeFolderToJson, self).__call__(version=version)

        folder_metadata.update({"is_folderish": True})
        result = folder_metadata

        include_items = self.request.form.get("include_items", include_items)
        include_items = boolean_value(include_items)
        if include_items:
            query = self._build_query()

            catalog = getToolByName(self.context, "portal_catalog")
            brains = catalog(query)

            batch = HypermediaBatch(self.request, brains)

            if not self.request.form.get("fullobjects"):
                result["@id"] = batch.canonical_url
            result["items_total"] = batch.items_total
            if batch.links:
                result["batching"] = batch.links

            if "fullobjects" in list(self.request.form):
                result["items"] = getMultiAdapter(
                    (brains, self.request), ISerializeToJson
                )(fullobjects=True)["items"]
            else:
                result["items"] = [
                    getMultiAdapter((brain, self.request), ISerializeToJsonSummary)()
                    for brain in batch
                ]
        return result



# -*- coding: utf-8 -*-
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.restapi.interfaces import IFieldDeserializer
from plone.restapi.testing import HAS_AT
from plone.restapi.testing import PLONE_RESTAPI_AT_INTEGRATION_TESTING
from zope.component import getMultiAdapter

import six
import unittest


class TestATFieldDeserializer(unittest.TestCase):

    layer = PLONE_RESTAPI_AT_INTEGRATION_TESTING

    def setUp(self):
        if not HAS_AT:
            raise unittest.SkipTest("Skip tests if Archetypes is not present")
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        setRoles(self.portal, TEST_USER_ID, ["Contributor"])

        self.portal.invokeFactory("ATTestDocument", id="doc1", title="Test Document")

    def deserialize(self, fieldname, value):
        field = self.portal.doc1.getField(fieldname)
        deserializer = getMultiAdapter(
            (field, self.portal.doc1, self.request), IFieldDeserializer
        )
        return deserializer(value)

    def test_string_field_deserialization_returns_string(self):
        value, kwargs = self.deserialize("testStringField", u"Käfer")
        self.assertTrue(isinstance(value, six.string_types), "Not a <basestring>")
        self.assertEqual(u"Käfer", value)

    def test_boolean_field_deserialization_returns_true(self):
        value, kwargs = self.deserialize("testBooleanField", True)
        self.assertTrue(isinstance(value, bool), "Not a <bool>")
        self.assertEqual(True, value)

    def test_boolean_field_deserialization_returns_false(self):
        value, kwargs = self.deserialize("testBooleanField", False)
        self.assertTrue(isinstance(value, bool), "Not a <bool>")
        self.assertEqual(False, value)

    def test_integer_field_deserialization_returns_integer_value(self):
        value, kwargs = self.deserialize("testIntegerField", 777)
        self.assertTrue(isinstance(value, int), "Not an <int>")
        self.assertEqual(777, value)

    def test_float_field_deserialization_returns_float_value(self):
        value, kwargs = self.deserialize("testFloatField", 1.5)
        self.assertTrue(isinstance(value, float), "Not a <float>")
        self.assertEqual(1.5, value)

    def test_fixedpoint_field_deserialization_returns_string(self):
        value, kwargs = self.deserialize("testFixedPointField", u"1.1")
        self.assertTrue(isinstance(value, six.string_types), "Not a <basestring>")
        self.assertEqual(u"1.1", value)

    def test_datetime_field_deserialization_returns_string(self):
        value, kwargs = self.deserialize(
            "testDateTimeField", u"2015-12-20T19:51:06.375Z"
        )
        self.assertTrue(isinstance(value, six.string_types), "Not a <basestring>")
        self.assertEqual(u"2015-12-20T19:51:06.375Z", value)

    def test_lines_field_deserialization_returns_string(self):
        value, kwargs = self.deserialize("testLinesField", u"line1\nline2")
        self.assertTrue(isinstance(value, six.string_types), "Not a <basestring>")
        self.assertEqual("line1\nline2", value)

    def test_lines_field_deserialization_returns_list(self):
        value, kwargs = self.deserialize("testLinesField", [u"line1", u"line2"])
        self.assertTrue(isinstance(value, list), "Not a <list>")
        self.assertEqual([u"line1", u"line2"], value)

    def test_file_field_deserialization_returns_string(self):
        value, kwargs = self.deserialize("testFileField", u"Spam and eggs!")
        self.assertTrue(isinstance(value, six.string_types), "Not a <basestring>")
        self.assertEqual(u"Spam and eggs!", value)

    def test_file_field_deserialization_returns_decoded_value(self):
        value, kwargs = self.deserialize(
            "testFileField", {u"data": u"U3BhbSBhbmQgZWdncyE=", u"encoding": u"base64"}
        )
        self.assertTrue(isinstance(value, str), "Not a <str>")
        self.assertEqual("Spam and eggs!", value)

    def test_file_field_deserialization_returns_mimetype_and_filename(self):
        value, kwargs = self.deserialize(
            "testFileField",
            {
                u"data": u"Spam and eggs!",
                u"filename": "doc.txt",
                u"content-type": "text/plain",
            },
        )
        self.assertTrue(isinstance(value, six.string_types), "Not a <basestring>")
        self.assertEqual(u"Spam and eggs!", value)
        self.assertEqual("text/plain", kwargs[u"mimetype"])
        self.assertEqual("doc.txt", kwargs[u"filename"])

    def test_text_field_deserialization_returns_string(self):
        value, kwargs = self.deserialize("testTextField", u"Käfer")
        self.assertTrue(isinstance(value, six.string_types), "Not a <basestring>")
        self.assertEqual(u"Käfer", value)

    def test_text_field_deserialization_returns_mimetype(self):
        value, kwargs = self.deserialize(
            "testTextField", {u"data": u"Käfer", u"content-type": "text/html"}
        )
        self.assertTrue(isinstance(value, six.string_types), "Not a <basestring>")
        self.assertEqual(u"Käfer", value)
        self.assertEqual("text/html", kwargs[u"mimetype"])

    def test_image_field_deserialization_returns_mimetype_and_filename(self):
        value, kwargs = self.deserialize(
            "testImageField",
            {
                u"data": u"R0lGODlhAQABAIAAAP///wAAACwAAAAAAQABAAACAkQBADs=",
                u"encoding": u"base64",
                u"filename": "image.gif",
                u"content-type": "image/gif",
            },
        )
        self.assertTrue(isinstance(value, six.string_types), "Not a <basestring>")
        self.assertTrue(value.startswith("GIF89a"))
        self.assertEqual("image/gif", kwargs[u"mimetype"])
        self.assertEqual("image.gif", kwargs[u"filename"])

    def test_blob_field_deserialization_returns_string(self):
        value, kwargs = self.deserialize("testBlobField", u"Spam and eggs!")
        self.assertTrue(isinstance(value, six.string_types), "Not a <basestring>")
        self.assertEqual(u"Spam and eggs!", value)

    def test_blob_field_deserialization_returns_mimetype_and_filename(self):
        value, kwargs = self.deserialize(
            "testBlobField",
            {
                u"data": u"Spam and eggs!",
                u"filename": "doc.txt",
                u"content-type": "text/plain",
            },
        )
        self.assertTrue(isinstance(value, six.string_types), "Not a <basestring>")
        self.assertEqual(u"Spam and eggs!", value)
        self.assertEqual("text/plain", kwargs[u"mimetype"])
        self.assertEqual("doc.txt", kwargs[u"filename"])

    def test_blobfile_field_deserialization_returns_mimetype_and_filename(self):
        value, kwargs = self.deserialize(
            "testBlobFileField",
            {
                u"data": u"Spam and eggs!",
                u"filename": "doc.txt",
                u"content-type": "text/plain",
            },
        )
        self.assertTrue(isinstance(value, six.string_types), "Not a <basestring>")
        self.assertEqual(u"Spam and eggs!", value)
        self.assertEqual("text/plain", kwargs[u"mimetype"])
        self.assertEqual("doc.txt", kwargs[u"filename"])

    def test_blobimage_field_deserialization_returns_mimetype_and_filename(self):
        value, kwargs = self.deserialize(
            "testBlobImageField",
            {
                u"data": u"R0lGODlhAQABAIAAAP///wAAACwAAAAAAQABAAACAkQBADs=",
                u"encoding": u"base64",
                u"filename": "image.gif",
                u"content-type": "image/gif",
            },
        )
        self.assertTrue(isinstance(value, six.string_types), "Not a <basestring>")
        self.assertTrue(value.startswith("GIF89a"))
        self.assertEqual("image/gif", kwargs[u"mimetype"])
        self.assertEqual("image.gif", kwargs[u"filename"])

    def test_query_field_deserialization_requests_list(self):
        query_data = [
            {
                "i": "portal_type",
                "o": "plone.app.querystring.operation.selection.is",
                "v": ["News Item"],
            },
            {
                "i": "path",
                "o": "plone.app.querystring.operation.string.path",
                "v": "/Plone/news",
            },
        ]
        value, kwargs = self.deserialize("testQueryField", query_data)
        self.assertTrue(isinstance(value, list), "Not a <list>")
        self.assertEqual(value, query_data)

    def test_reference_field_deserialization_returns_uid_in_list(self):
        value, kwargs = self.deserialize(
            "testReferenceField", u"0fc0dac495034b869b3b90c9179499a9"
        )
        self.assertTrue(isinstance(value, list), "Not a <basestring>")
        self.assertEqual([u"0fc0dac495034b869b3b90c9179499a9"], value)

    def test_reference_field_deserialization_returns_uids(self):
        value, kwargs = self.deserialize(
            "testReferenceField",
            [u"0fc0dac495034b869b3b90c9179499a9", u"984c22058343453f997ef9e9de1e8136"],
        )
        self.assertTrue(isinstance(value, list), "Not a <list>")
        self.assertIn(u"0fc0dac495034b869b3b90c9179499a9", value)
        self.assertIn(u"984c22058343453f997ef9e9de1e8136", value)

    def test_reference_field_deserialization_returns_object_in_list(self):
        doc2 = self.portal[
            self.portal.invokeFactory(
                "ATTestDocument", id="doc2", title="Referenced Document"
            )
        ]
        value, kwargs = self.deserialize(
            "testReferenceField", six.text_type(doc2.absolute_url())
        )
        self.assertEqual(doc2, value[0])

    def test_reference_field_deserialization_returns_objects(self):
        doc2 = self.portal[
            self.portal.invokeFactory(
                "ATTestDocument", id="doc2", title="Referenced Document"
            )
        ]
        doc3 = self.portal[
            self.portal.invokeFactory(
                "ATTestDocument", id="doc3", title="Referenced Document"
            )
        ]
        value, kwargs = self.deserialize(
            "testReferenceField",
            [six.text_type(doc2.absolute_url()), six.text_type(doc3.absolute_url())],
        )
        self.assertEqual(doc2, value[0])
        self.assertEqual(doc3, value[1])



# -*- coding: utf-8 -*-
from plone import api
from plone.app.discussion.interfaces import IConversation
from plone.app.discussion.interfaces import IDiscussionSettings
from plone.app.discussion.interfaces import IReplies
from plone.registry.interfaces import IRegistry
from plone.restapi.interfaces import ISerializeToJson
from plone.restapi.testing import PLONE_RESTAPI_DX_INTEGRATION_TESTING
from unittest import TestCase
from zope.component import createObject
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.component import queryUtility


class TestCommentsSerializers(TestCase):
    layer = PLONE_RESTAPI_DX_INTEGRATION_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.portal_url = self.portal.absolute_url()

        # Allow discussion
        registry = getUtility(IRegistry)
        settings = registry.forInterface(IDiscussionSettings, check=False)
        settings.globally_enabled = True
        settings.edit_comment_enabled = True
        settings.delete_own_comment_enabled = True

        # doc with comments
        self.doc = api.content.create(
            container=self.portal,
            type="Document",
            id="doc_with_comments",
            title="Document with comments",
            allow_discussion=True,
        )
        self.conversation = IConversation(self.doc)
        self.replies = IReplies(self.conversation)
        comment = createObject("plone.Comment")
        comment.text = "Comment"
        self.comment = self.replies[self.replies.addComment(comment)]

        comment = createObject("plone.Comment")
        comment.text = "Comment 2"
        self.replies.addComment(comment)

    def test_conversation(self):
        serializer = getMultiAdapter(
            (self.conversation, self.request), ISerializeToJson
        )

        output = serializer()
        self.assertEqual(set(output), set(["@id", "items_total", "items"]))

    def test_conversation_batched(self):
        self.request.form["b_size"] = 1
        serializer = getMultiAdapter(
            (self.conversation, self.request), ISerializeToJson
        )

        output = serializer()
        self.assertIn("batching", output)

    def test_comment(self):
        serializer = getMultiAdapter((self.comment, self.request), ISerializeToJson)

        output = serializer()

        expected = [
            "@id",
            "@type",
            "@parent",
            "comment_id",
            "in_reply_to",
            "text",
            "user_notification",
            "author_username",
            "author_name",
            "creation_date",
            "modification_date",
            "is_editable",
            "is_deletable",
        ]
        self.assertEqual(set(output), set(expected))

        self.assertEqual(set(output["text"]), set(["data", "mime-type"]))

    def test_comment_with_mimetype_text_plain(self):

        self.conversation = IConversation(self.doc)
        self.replies = IReplies(self.conversation)
        comment = createObject("plone.Comment")
        comment.text = "Hey, I am plain text!"
        comment.mime_type = "text/plain"
        self.comment = self.replies[self.replies.addComment(comment)]

        serializer = getMultiAdapter((self.comment, self.request), ISerializeToJson)

        # serializer should return HTML with a clickable link
        self.assertEqual(
            "Hey, I am plain text!", serializer()["text"]["data"],
        )
        # serializer should return mimetype = text/x-web-intelligent
        self.assertEqual("text/plain", serializer()["text"]["mime-type"])

    def test_comment_with_mimetype_intelligent_text(self):
        # Set text transform to intelligent text
        registry = queryUtility(IRegistry)
        settings = registry.forInterface(IDiscussionSettings, check=False)
        settings.text_transform = "text/x-web-intelligent"

        self.conversation = IConversation(self.doc)
        self.replies = IReplies(self.conversation)
        comment = createObject("plone.Comment")
        comment.text = "Go to https://www.plone.org"
        comment.mime_type = "text/x-web-intelligent"
        self.comment = self.replies[self.replies.addComment(comment)]

        serializer = getMultiAdapter((self.comment, self.request), ISerializeToJson)

        # serializer should return HTML with a clickable link
        self.assertEqual(
            'Go to <a href="https://www.plone.org" '
            + 'rel="nofollow">https://www.plone.org</a>',
            serializer()["text"]["data"],
        )
        # serializer should return mimetype = text/html
        self.assertEqual("text/html", serializer()["text"]["mime-type"])

    def test_comment_with_mimetype_html(self):
        # Set text transform to text/html
        registry = queryUtility(IRegistry)
        settings = registry.forInterface(IDiscussionSettings, check=False)
        settings.text_transform = "text/html"

        self.conversation = IConversation(self.doc)
        self.replies = IReplies(self.conversation)
        comment = createObject("plone.Comment")
        comment.text = "Go to <a href='https://www.plone.org'>Plone</a>"
        comment.mime_type = "text/html"
        self.comment = self.replies[self.replies.addComment(comment)]

        serializer = getMultiAdapter((self.comment, self.request), ISerializeToJson)

        # serializer should return HTML
        self.assertEqual(
            'Go to <a href="https://www.plone.org">Plone</a>',
            serializer()["text"]["data"],
        )
        # serializer should return mimetype = text/html
        self.assertEqual("text/html", serializer()["text"]["mime-type"])



# -*- coding: utf-8 -*-
from plone.restapi.interfaces import ISerializeToJson
from plone.restapi.interfaces import IZCatalogCompatibleQuery
from Products.CMFCore.utils import getToolByName
from zope.component import getMultiAdapter


class SearchHandler(object):
    """Executes a catalog search based on a query dict, and returns
    JSON compatible results.
    """

    def __init__(self, context, request):
        self.context = context
        self.request = request
        self.catalog = getToolByName(self.context, "portal_catalog")

    def _parse_query(self, query):
        catalog_compatible_query = getMultiAdapter(
            (self.context, self.request), IZCatalogCompatibleQuery
        )(query)
        return catalog_compatible_query

    def _constrain_query_by_path(self, query):
        """If no 'path' query was supplied, restrict search to current
        context and its children by adding a path constraint.

        The following cases can happen here:
        - No 'path' parameter at all
        - 'path' query dict with options, but no actual 'query' inside
        - 'path' supplied as a string
        - 'path' supplied as a complete query dict
        """
        if "path" not in query:
            query["path"] = {}

        if isinstance(query["path"], str) or isinstance(query["path"], list):
            query["path"] = {"query": query["path"]}

        # If this is accessed through a VHM the client does not know
        # the complete physical path of an object. But the path index
        # indexes the complete physical path. Complete the path.
        vhm_physical_path = self.request.get("VirtualRootPhysicalPath")
        if vhm_physical_path:
            path = query["path"].get("query")
            if path:
                if isinstance(path, str):
                    path = path.lstrip("/")
                    full_path = "/".join(vhm_physical_path + (path,))
                    query["path"]["query"] = full_path
                if isinstance(path, list):
                    full_paths = []
                    for p in path:
                        p = p.lstrip("/")
                        full_path = "/".join(vhm_physical_path + (p,))
                        full_paths.append(full_path)
                    query["path"]["query"] = full_paths

        if isinstance(query["path"], dict) and "query" not in query["path"]:
            # We either had no 'path' parameter at all, or an incomplete
            # 'path' query dict (with just ExtendedPathIndex options (like
            # 'depth'), but no actual path 'query' in it).
            #
            # In either case, we'll prefill with the context's path
            path = "/".join(self.context.getPhysicalPath())
            query["path"]["query"] = path

    def search(self, query=None):
        if query is None:
            query = {}
        if "fullobjects" in query:
            fullobjects = True
            del query["fullobjects"]
        else:
            fullobjects = False

        metadata_fields = query.pop("metadata_fields", [])
        if not isinstance(metadata_fields, list):
            metadata_fields = [metadata_fields]

        self._constrain_query_by_path(query)
        query = self._parse_query(query)

        lazy_resultset = self.catalog.searchResults(query)
        results = getMultiAdapter((lazy_resultset, self.request), ISerializeToJson)(
            metadata_fields=metadata_fields, fullobjects=fullobjects
        )

        return results



# -*- coding: utf-8 -*-
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.restapi.interfaces import IDeserializeFromJson
from plone.restapi.testing import HAS_AT
from plone.restapi.testing import PLONE_RESTAPI_AT_INTEGRATION_TESTING
from plone.restapi.tests.mixin_ordering import OrderingMixin
from six.moves import range
from zExceptions import BadRequest
from zope.component import adapter
from zope.component import getMultiAdapter
from zope.component import provideHandler
from zope.component import provideSubscriptionAdapter
from zope.interface import implementer

import unittest


if HAS_AT:
    from plone.restapi.deserializer.atcontent import ValidationRequest
    from Products.Archetypes.interfaces import IBaseObject
    from Products.Archetypes.interfaces import IObjectEditedEvent
    from Products.Archetypes.interfaces import IObjectInitializedEvent
    from Products.Archetypes.interfaces import IObjectPostValidation
    from Products.Archetypes.interfaces import IObjectPreValidation


class TestATContentDeserializer(unittest.TestCase, OrderingMixin):

    layer = PLONE_RESTAPI_AT_INTEGRATION_TESTING

    def setUp(self):
        if not HAS_AT:
            raise unittest.SkipTest("Skip tests if Archetypes is not present")
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        setRoles(self.portal, TEST_USER_ID, ["Contributor"])

        self.doc1 = self.portal[
            self.portal.invokeFactory(
                "ATTestDocument", id="doc1", title="Test Document"
            )
        ]

        # ordering setup
        self.folder = self.portal[
            self.portal.invokeFactory("ATTestFolder", id="folder1", title="Test folder")
        ]

        for x in range(1, 10):
            self.folder.invokeFactory(
                "ATTestDocument", id="doc" + str(x), title="Test doc " + str(x)
            )

    def deserialize(self, body="{}", validate_all=False, create=False, context=None):
        context = context or self.doc1
        self.request["BODY"] = body
        deserializer = getMultiAdapter((context, self.request), IDeserializeFromJson)
        return deserializer(validate_all=validate_all, create=create)

    def test_deserializer_ignores_readonly_fields(self):
        self.doc1.getField("testReadonlyField").set(self.doc1, "Readonly")
        self.deserialize(body='{"testReadonlyField": "Changed"}')
        self.assertEqual("Readonly", self.doc1.getTestReadonlyField())

    def test_deserializer_updates_field_value(self):
        self.deserialize(body='{"testStringField": "Updated"}')
        self.assertEqual("Updated", self.doc1.getTestStringField())

    def test_deserializer_validates_content(self):
        with self.assertRaises(BadRequest) as cm:
            self.deserialize(body='{"testURLField": "Not an URL"}')
        self.assertEqual(
            u"Validation failed(isURL): 'Not an URL' is not a valid url.",
            cm.exception.args[0][0]["message"],
        )

    def test_deserializer_clears_creation_flag(self):
        self.doc1.markCreationFlag()
        self.deserialize(body='{"testStringField": "Updated"}', create=True)
        self.assertFalse(self.doc1.checkCreationFlag(), "Creation flag not cleared")

    def test_deserializer_notifies_object_initialized(self):
        def handler(obj, event):
            obj._handler_called = True

        provideHandler(handler, (IBaseObject, IObjectInitializedEvent))
        self.doc1.markCreationFlag()
        self.deserialize(body='{"testStringField": "Updated"}')
        self.assertTrue(
            getattr(self.doc1, "_handler_called", False),
            "IObjectInitializedEvent not notified",
        )

    def test_deserializer_notifies_object_edited(self):
        def handler(obj, event):
            obj._handler_called = True

        provideHandler(handler, (IBaseObject, IObjectEditedEvent))
        self.doc1.unmarkCreationFlag()
        self.deserialize(body='{"testStringField": "Updated"}')
        self.assertTrue(
            getattr(self.doc1, "_handler_called", False),
            "IObjectEditedEvent not notified",
        )

    def test_deserializer_raises_if_required_value_is_missing(self):
        with self.assertRaises(BadRequest) as cm:
            self.deserialize(body='{"testStringField": "My Value"}', validate_all=True)
        self.assertEqual(
            u"TestRequiredField is required, please correct.",
            cm.exception.args[0][0]["message"],
        )

    def test_deserializer_succeeds_if_required_value_is_provided(self):
        self.deserialize(body='{"testRequiredField": "My Value"}', validate_all=True)
        self.assertEqual(u"My Value", self.portal.doc1.getTestRequiredField())

    def test_post_validation(self):
        @implementer(IObjectPostValidation)
        @adapter(IBaseObject)
        class PostValidator(object):
            def __init__(self, context):
                self.context = context

            def __call__(self, request):
                return {"post": "post_validation_error"}

        provideSubscriptionAdapter(PostValidator)

        with self.assertRaises(BadRequest) as cm:
            self.deserialize(
                body='{"testRequiredField": "My Value"}', validate_all=True
            )

        self.assertEqual("post_validation_error", cm.exception.args[0][0]["message"])

    def test_pre_validation(self):
        @implementer(IObjectPreValidation)
        @adapter(IBaseObject)
        class PreValidator(object):
            def __init__(self, context):
                self.context = context

            def __call__(self, request):
                return {"pre": "pre_validation_error"}

        provideSubscriptionAdapter(PreValidator)

        with self.assertRaises(BadRequest) as cm:
            self.deserialize(
                body='{"testRequiredField": "My Value"}', validate_all=True
            )

        self.assertEqual("pre_validation_error", cm.exception.args[0][0]["message"])

    def test_set_layout(self):
        current_layout = self.doc1.getLayout()
        self.assertNotEqual(current_layout, "my_new_layout")
        self.deserialize(body='{"layout": "my_new_layout"}')
        self.assertEqual("my_new_layout", self.doc1.getLayout())


class TestValidationRequest(unittest.TestCase):

    layer = PLONE_RESTAPI_AT_INTEGRATION_TESTING

    def setUp(self):
        if not HAS_AT:
            raise unittest.SkipTest("Skip tests if Archetypes is not present")
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Contributor"])
        self.doc1 = self.portal[
            self.portal.invokeFactory(
                "ATTestDocument", id="doc1", title="Test Document"
            )
        ]
        self.request = ValidationRequest(self.layer["request"], self.doc1)

    def test_value_from_validation_request_using_key_access(self):
        self.assertEqual("Test Document", self.request["title"])

    def test_value_from_validation_request_using_get(self):
        self.assertEqual("Test Document", self.request.get("title"))

    def test_value_from_validation_request_form_using_key_access(self):
        self.assertEqual("Test Document", self.request.form["title"])

    def test_value_from_validation_request_form_using_get(self):
        self.assertEqual("Test Document", self.request.form.get("title"))

    def test_validation_request_contains_key(self):
        self.assertIn("title", self.request)

    def test_validation_request_form_contains_key(self):
        self.assertIn("title", self.request.form)

    def test_validation_request_key_access_raises_keyerror(self):
        with self.assertRaises(KeyError):
            self.request["foo"]

    def test_validation_request_get_returns_default_value(self):
        self.assertEqual(None, self.request.get("foo"))
        marker = object()
        self.assertEqual(marker, self.request.get("foo", marker))

    def test_validation_request_form_key_access_raises_keyerror(self):
        with self.assertRaises(KeyError):
            self.request.form["foo"]

    def test_validation_request_form_get_returns_default_value(self):
        self.assertEqual(None, self.request.form.get("foo"))
        marker = object()
        self.assertEqual(marker, self.request.form.get("foo", marker))

    def test_value_from_real_request_using_key_access(self):
        self.assertEqual("GET", self.request["REQUEST_METHOD"])

    def test_value_form_real_request_using_get(self):
        self.assertEqual("GET", self.request.get("REQUEST_METHOD"))



# -*- coding: utf-8 -*-
# XXX: EXPERIMENTAL!!!
# This is an experimental feature meant for use in Volto only!
# This code is likely to change in the future, even within minor releases.
# We will make sure plone.restapi latest always works with the latest Volto release.
# This code is planned to being refactored into CMFPlone 6.0 as soon as Volto 4 final is out.
# <tisto@plone.org>
from plone.restapi.behaviors import IBlocks
from plone.indexer.decorator import indexer
from plone.app.contenttypes.indexers import SearchableText
import six


def _extract_text(block):
    result = ""
    for paragraph in block.get("text").get("blocks"):
        text = paragraph["text"]
        if six.PY2:
            if isinstance(text, six.text_type):
                text = text.encode("utf-8", "replace")
            if text:
                result = " ".join((result, text))
        else:
            result = " ".join((result, text))
    return result


@indexer(IBlocks)
def SearchableText_blocks(obj):
    std_text = SearchableText(obj)
    blocks = obj.blocks
    blocks_text = [
        _extract_text(blocks[block_uid])
        for block_uid in obj.blocks
        if blocks[block_uid].get("@type", "") == "text"
    ]
    blocks_text.append(std_text)
    text = " ".join(blocks_text)
    return text



# -*- coding: utf-8 -*-
from DateTime import DateTime
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.restapi.interfaces import ISerializeToJson
from plone.restapi.testing import HAS_AT
from plone.restapi.testing import PLONE_RESTAPI_AT_INTEGRATION_TESTING
from zope.component import getMultiAdapter

import unittest


class TestATContentSerializer(unittest.TestCase):

    layer = PLONE_RESTAPI_AT_INTEGRATION_TESTING

    def setUp(self):
        if not HAS_AT:
            raise unittest.SkipTest("Skip tests if Archetypes is not present")
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        self.doc1 = self.portal[
            self.portal.invokeFactory(
                "Collection", id="collection", title="Test Collection"
            )
        ]

        self.doc1.setCreationDate(DateTime("2016-01-21T01:14:48+00:00"))
        self.doc1.setModificationDate(DateTime("2016-01-21T01:24:11+00:00"))
        self.doc1._setUID("76644b6611ab44c6881efd9cb17db12e")
        query_data = [
            {
                "i": "portal_type",
                "o": "plone.app.querystring.operation.selection.is",
                "v": ["ATTestFolder"],
            },
            {
                "i": "path",
                "o": "plone.app.querystring.operation.string.path",
                "v": "/plone/folder",
            },
        ]

        self.doc1.setQuery(query_data)
        self.doc1.setSort_on("created")

    def serialize(self, obj, fullobjects=False):
        if fullobjects:
            self.request.form["fullobjects"] = 1

        serializer = getMultiAdapter((obj, self.request), ISerializeToJson)
        return serializer()

    def test_serializer_includes_collection_items(self):
        folder = self.portal[
            self.portal.invokeFactory("ATTestFolder", id="folder", title="Test Folder")
        ]
        folder.invokeFactory("ATTestFolder", id="subfolder-1", title="Subfolder 1")
        folder.invokeFactory("ATTestFolder", id="subfolder-2", title="Subfolder 2")
        folder.invokeFactory("ATTestDocument", id="doc", title="A Document")
        obj = self.serialize(self.doc1)
        self.assertIn("items", obj)
        items = obj["items"]
        items = sorted(items, key=lambda item: item[u"@id"])
        self.assertDictEqual(
            {
                u"@id": u"http://nohost/plone/folder",
                u"@type": u"ATTestFolder",
                u"description": u"",
                u"title": u"Test Folder",
                "review_state": "private",
            },
            items[0],
        )
        self.assertDictEqual(
            {
                u"@id": u"http://nohost/plone/folder/subfolder-1",
                u"@type": u"ATTestFolder",
                u"description": u"",
                u"title": u"Subfolder 1",
                "review_state": "private",
            },
            items[1],
        )
        self.assertDictEqual(
            {
                u"@id": u"http://nohost/plone/folder/subfolder-2",
                u"@type": u"ATTestFolder",
                u"description": u"",
                u"title": u"Subfolder 2",
                "review_state": "private",
            },
            items[2],
        )

    def test_serializer_includes_collection_fullobjects(self):
        """ when using the fullobjects parameter, the collection needs to
            serialize its contents with the standard object serializer"""
        folder = self.portal[
            self.portal.invokeFactory("ATTestFolder", id="folder", title="Test Folder")
        ]
        folder.invokeFactory("ATTestFolder", id="subfolder-1", title="Subfolder 1")
        folder.invokeFactory("ATTestFolder", id="subfolder-2", title="Subfolder 2")
        folder.invokeFactory("ATTestDocument", id="doc", title="A Document")
        obj = self.serialize(self.doc1, fullobjects=True)
        self.assertIn("items", obj)
        items = obj["items"]
        items = sorted(items, key=lambda item: item[u"@id"])
        self.assertIn("UID", items[0])
        self.assertEquals(items[0]["id"], "folder")

        self.assertIn("UID", items[1])
        self.assertEquals(items[1]["id"], "subfolder-1")

        self.assertIn("UID", items[2])
        self.assertEquals(items[2]["id"], "subfolder-2")



# -*- coding: utf-8 -*-
from plone.app.multilingual.interfaces import ITranslatable
from plone.app.multilingual.interfaces import ITranslationManager
from plone.restapi.deserializer import json_body
from plone.restapi.interfaces import IExpandableElement
from plone.restapi.services import Service
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.interfaces import ILanguage
from zope.component import adapter
from zope.component import getMultiAdapter
from zope.interface import alsoProvides
from zope.interface import implementer
from zope.interface import Interface

import plone.protect.interfaces
import six


@implementer(IExpandableElement)
@adapter(ITranslatable, Interface)
class Translations(object):
    def __init__(self, context, request):
        self.context = context
        self.request = request

    def __call__(self, expand=False):
        result = {
            "translations": {
                "@id": "{}/@translations".format(self.context.absolute_url())
            }
        }
        if not expand:
            return result

        translations = []
        manager = ITranslationManager(self.context)
        for language, translation in manager.get_restricted_translations().items():
            if language != ILanguage(self.context).get_language():
                translations.append(
                    {"@id": translation.absolute_url(), "language": language}
                )

        result["translations"]["items"] = translations
        return result


class TranslationInfo(Service):
    """ Get translation information
    """

    def reply(self):
        translations = Translations(self.context, self.request)
        return translations(expand=True)["translations"]


class LinkTranslations(Service):
    """ Link two content objects as translations of each other
    """

    def __init__(self, context, request):
        super(LinkTranslations, self).__init__(context, request)
        self.portal = getMultiAdapter(
            (self.context, self.request), name="plone_portal_state"
        ).portal()
        self.portal_url = self.portal.absolute_url()
        self.catalog = getToolByName(self.context, "portal_catalog")

    def reply(self):
        # Disable CSRF protection
        if "IDisableCSRFProtection" in dir(plone.protect.interfaces):
            alsoProvides(self.request, plone.protect.interfaces.IDisableCSRFProtection)

        data = json_body(self.request)
        id_ = data.get("id", None)
        if id_ is None:
            self.request.response.setStatus(400)
            return dict(
                error=dict(type="BadRequest", message="Missing content id to link to")
            )

        target = self.get_object(id_)
        if target is None:
            self.request.response.setStatus(400)
            return dict(error=dict(type="BadRequest", message="Content does not exist"))

        target_language = ILanguage(target).get_language()
        manager = ITranslationManager(self.context)
        current_translation = manager.get_translation(target_language)
        if current_translation is not None:
            self.request.response.setStatus(400)
            return dict(
                error=dict(
                    type="BadRequest",
                    message="Already translated into language {}".format(
                        target_language
                    ),
                )
            )

        manager.register_translation(target_language, target)
        self.request.response.setStatus(201)
        self.request.response.setHeader("Location", self.context.absolute_url())
        return {}

    def get_object(self, key):
        if isinstance(key, six.string_types):
            if key.startswith(self.portal_url):
                # Resolve by URL
                key = key[len(self.portal_url) + 1 :]
                if six.PY2:
                    key = key.encode("utf8")
                return self.portal.restrictedTraverse(key, None)
            elif key.startswith("/"):
                if six.PY2:
                    key = key.encode("utf8")
                # Resolve by path
                return self.portal.restrictedTraverse(key.lstrip("/"), None)
            else:
                # Resolve by UID
                brain = self.catalog(UID=key)
                if brain:
                    return brain[0].getObject()


class UnlinkTranslations(Service):
    """ Unlink the translations for a content object
    """

    def reply(self):
        # Disable CSRF protection
        if "IDisableCSRFProtection" in dir(plone.protect.interfaces):
            alsoProvides(self.request, plone.protect.interfaces.IDisableCSRFProtection)

        data = json_body(self.request)
        manager = ITranslationManager(self.context)
        language = data.get("language", None)
        if language is None:
            self.request.response.setStatus(400)
            return dict(
                error=dict(
                    type="BadRequest",
                    message="You need to provide the language to unlink",
                )
            )

        if language not in list(manager.get_translations()):
            self.request.response.setStatus(400)
            return dict(
                error=dict(
                    type="BadRequest",
                    message="This objects is not translated into {}".format(language),
                )
            )

        manager.remove_translation(language)
        return self.reply_no_content()



# -*- coding: utf-8 -*-
from plone.restapi.interfaces import IExpandableElement
from plone.restapi.services import Service
from zope.component import adapter
from zope.component import getMultiAdapter
from zope.interface import implementer
from zope.interface import Interface


@implementer(IExpandableElement)
@adapter(Interface, Interface)
class Breadcrumbs(object):
    def __init__(self, context, request):
        self.context = context
        self.request = request

    def __call__(self, expand=False):
        result = {
            "breadcrumbs": {
                "@id": "{}/@breadcrumbs".format(self.context.absolute_url())
            }
        }
        if not expand:
            return result

        breadcrumbs_view = getMultiAdapter(
            (self.context, self.request), name="breadcrumbs_view"
        )
        items = []
        for crumb in breadcrumbs_view.breadcrumbs():
            items.append({"title": crumb["Title"], "@id": crumb["absolute_url"]})

        result["breadcrumbs"]["items"] = items
        return result


class BreadcrumbsGet(Service):
    def reply(self):
        breadcrumbs = Breadcrumbs(self.context, self.request)
        return breadcrumbs(expand=True)["breadcrumbs"]



# -*- coding: utf-8 -*-
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.restapi.testing import PLONE_RESTAPI_DX_FUNCTIONAL_TESTING
from plone.restapi.testing import RelativeSession

import unittest


class TestQuerystringEndpoint(unittest.TestCase):

    layer = PLONE_RESTAPI_DX_FUNCTIONAL_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)

    def tearDown(self):
        self.api_session.close()

    def test_endpoint_lists_indexes(self):
        response = self.api_session.get("/@querystring")

        self.assertEqual(response.status_code, 200)
        self.assertIn("indexes", response.json())
        self.assertIn("isFolderish", response.json()["indexes"])

    def test_endpoint_lists_sortable_indexes(self):
        response = self.api_session.get("/@querystring")

        self.assertEqual(response.status_code, 200)
        self.assertIn("sortable_indexes", response.json())
        self.assertIn("sortable_title", response.json()["sortable_indexes"])

    def test_endpoint_shows_field_config(self):
        response = self.api_session.get("/@querystring")

        self.assertEqual(response.status_code, 200)
        indexes = response.json()["indexes"]
        idx = indexes["Title"]

        expected_field_config = {
            u"description": u"Text search of an item's title",
            u"enabled": True,
            u"group": u"Text",
            u"operations": [u"plone.app.querystring.operation.string.contains"],
            u"operators": {
                u"plone.app.querystring.operation.string.contains": {
                    u"description": None,
                    u"operation": u"plone.app.querystring.queryparser._contains",
                    u"title": u"Contains",
                    u"widget": u"StringWidget",
                }
            },
            u"sortable": False,
            u"title": u"Title",
            u"values": {},
            u"vocabulary": None,
        }
        self.assertEqual(expected_field_config, idx)

    def test_endpoint_inlines_vocabularies(self):
        response = self.api_session.get("/@querystring")

        self.assertEqual(response.status_code, 200)
        indexes = response.json()["indexes"]
        idx = indexes["review_state"]

        self.assertDictContainsSubset(
            {
                "title": u"Review state",
                "vocabulary": u"plone.app.vocabularies.WorkflowStates",
            },
            idx,
        )

        expected_vocab_values = {
            u"external": {u"title": u"Externally visible [external]"},
            u"internal": {u"title": u"Internal draft [internal]"},
            u"internally_published": {
                u"title": u"Internally published [internally_published]"
            },
            u"pending": {u"title": u"Pending [pending]"},
            u"private": {u"title": u"Private [private]"},
            u"published": {u"title": u"Published with accent \xe9 [published]"},
            u"visible": {u"title": u"Public draft [visible]"},
        }
        self.assertTrue(
            all(elem in idx["values"].items() for elem in expected_vocab_values.items())
        )

    def test_endpoint_inlines_operators(self):
        response = self.api_session.get("/@querystring")

        self.assertEqual(response.status_code, 200)
        indexes = response.json()["indexes"]
        idx = indexes["isDefaultPage"]

        self.assertDictContainsSubset(
            {
                "title": u"Default Page",
                "operations": [
                    u"plone.app.querystring.operation.boolean.isTrue",
                    u"plone.app.querystring.operation.boolean.isFalse",
                ],
            },
            idx,
        )

        expected_operators = {
            u"plone.app.querystring.operation.boolean.isFalse": {
                u"description": None,
                u"operation": u"plone.app.querystring.queryparser._isFalse",
                u"title": u"No",
                u"widget": None,
            },
            u"plone.app.querystring.operation.boolean.isTrue": {
                u"description": None,
                u"operation": u"plone.app.querystring.queryparser._isTrue",
                u"title": u"Yes",
                u"widget": None,
            },
        }
        self.assertEqual(expected_operators, idx["operators"])

    def test_endpoint_includes_widgets_for_operators(self):
        response = self.api_session.get("/@querystring")

        self.assertEqual(response.status_code, 200)
        indexes = response.json()["indexes"]
        idx = indexes["getObjPositionInParent"]

        self.assertDictContainsSubset(
            {
                "title": u"Order in folder",
                "operations": [
                    "plone.app.querystring.operation.int.is",
                    "plone.app.querystring.operation.int.lessThan",
                    "plone.app.querystring.operation.int.largerThan",
                ],
            },
            idx,
        )

        ops = idx["operators"]

        self.assertEqual(
            {
                "description": None,
                "operation": "plone.app.querystring.queryparser._intLargerThan",
                "title": "Larger than",
                "widget": "StringWidget",
            },
            ops["plone.app.querystring.operation.int.largerThan"],
        )



# -*- coding: utf-8 -*-
from plone.restapi.interfaces import IFieldDeserializer
from plone.restapi.services.content.tus import TUSUpload
from Products.Archetypes.interfaces import IBaseObject
from Products.Archetypes.interfaces.field import IField
from Products.Archetypes.interfaces.field import IFileField
from Products.Archetypes.interfaces.field import IReferenceField
from zope.component import adapter
from zope.component import getMultiAdapter
from zope.interface import implementer
from zope.publisher.interfaces.browser import IBrowserRequest

import pkg_resources

try:
    pkg_resources.get_distribution("plone.app.blob")
except pkg_resources.DistributionNotFound:
    HAS_BLOB = False
else:
    HAS_BLOB = True
    from plone.app.blob.interfaces import IBlobField


@implementer(IFieldDeserializer)
@adapter(IField, IBaseObject, IBrowserRequest)
class DefaultFieldDeserializer(object):
    def __init__(self, field, context, request):
        self.field = field
        self.context = context
        self.request = request

    def __call__(self, value):
        return value, {}


@implementer(IFieldDeserializer)
@adapter(IFileField, IBaseObject, IBrowserRequest)
class FileFieldDeserializer(DefaultFieldDeserializer):
    def __call__(self, value):
        kwargs = {}
        if isinstance(value, dict):
            if u"content-type" in value:
                kwargs[u"mimetype"] = value[u"content-type"].encode("utf8")
            if u"filename" in value:
                kwargs[u"filename"] = value[u"filename"].encode("utf8")
            if u"encoding" in value:
                value = value.get("data", "").decode(value[u"encoding"])
            else:
                value = value.get("data", "")
        elif isinstance(value, TUSUpload):
            metadata = value.metadata()
            if "content-type" in metadata:
                kwargs[u"mimetype"] = metadata["content-type"]
            if "filename" in metadata:
                kwargs[u"filename"] = metadata["filename"]
            value = value.open()

        return value, kwargs


if HAS_BLOB:

    @implementer(IFieldDeserializer)
    @adapter(IBlobField, IBaseObject, IBrowserRequest)
    class BlobFieldDeserializer(FileFieldDeserializer):
        pass


@implementer(IFieldDeserializer)
@adapter(IReferenceField, IBaseObject, IBrowserRequest)
class ReferenceFieldDeserializer(DefaultFieldDeserializer):
    def __call__(self, value):
        portal = getMultiAdapter(
            (self.context, self.request), name="plone_portal_state"
        ).portal()
        portal_url = portal.absolute_url()

        if not isinstance(value, list):
            value = [value]

        for i, v in enumerate(value):
            # Resolve references given by URL
            if v.startswith(portal_url):
                path = v[len(portal_url) + 1 :].encode("utf8")
                value[i] = portal.unrestrictedTraverse(path, None)

        return value, {}



# -*- coding: utf-8 -*-
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.restapi.testing import PLONE_RESTAPI_DX_FUNCTIONAL_TESTING
from plone.restapi.testing import RelativeSession
from Products.CMFCore.permissions import SetOwnPassword
from Products.CMFCore.utils import getToolByName
from Products.MailHost.interfaces import IMailHost
from zope.component import getAdapter
from zope.component import getUtility

import transaction
import unittest


try:
    from Products.CMFPlone.interfaces import ISecuritySchema
except ImportError:
    from plone.app.controlpanel.security import ISecuritySchema


class TestUsersEndpoint(unittest.TestCase):

    layer = PLONE_RESTAPI_DX_FUNCTIONAL_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        self.mailhost = getUtility(IMailHost)

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)
        self.anon_api_session = RelativeSession(self.portal_url)
        self.anon_api_session.headers.update({"Accept": "application/json"})

        properties = {
            "email": "noam.chomsky@example.com",
            "username": "noamchomsky",
            "fullname": "Noam Avram Chomsky",
            "home_page": "web.mit.edu/chomsky",
            "description": "Professor of Linguistics",
            "location": "Cambridge, MA",
        }
        api.user.create(
            email="noam.chomsky@example.com",
            username="noam",
            properties=properties,
            password=u"password",
        )
        properties = {
            "email": "otheruser@example.com",
            "username": "otheruser",
            "fullname": "Other user",
        }
        api.user.create(
            email="otheruser@example.com",
            username="otheruser",
            properties=properties,
            password=u"otherpassword",
        )
        transaction.commit()

    def tearDown(self):
        self.api_session.close()
        self.anon_api_session.close()

    def test_list_users(self):
        response = self.api_session.get("/@users")

        self.assertEqual(200, response.status_code)
        self.assertEqual(4, len(response.json()))
        user_ids = [user["id"] for user in response.json()]
        self.assertIn("admin", user_ids)
        self.assertIn("test_user_1_", user_ids)
        self.assertIn("noam", user_ids)
        noam = [x for x in response.json() if x.get("username") == "noam"][0]
        self.assertEqual("noam", noam.get("id"))
        self.assertEqual(self.portal.absolute_url() + "/@users/noam", noam.get("@id"))
        self.assertEqual("noam.chomsky@example.com", noam.get("email"))
        self.assertEqual("Noam Avram Chomsky", noam.get("fullname"))
        self.assertEqual("web.mit.edu/chomsky", noam.get("home_page"))  # noqa
        self.assertEqual("Professor of Linguistics", noam.get("description"))  # noqa
        self.assertEqual("Cambridge, MA", noam.get("location"))

    def test_list_users_without_being_manager(self):
        noam_api_session = RelativeSession(self.portal_url)
        noam_api_session.headers.update({"Accept": "application/json"})
        noam_api_session.auth = ("noam", "password")

        response = noam_api_session.get("/@users")
        self.assertEqual(response.status_code, 401)
        noam_api_session.close()

    def test_list_users_as_anonymous(self):

        response = self.anon_api_session.get("/@users")
        self.assertEqual(response.status_code, 401)

    def test_add_user(self):
        response = self.api_session.post(
            "/@users",
            json={
                "username": "howard",
                "email": "howard.zinn@example.com",
                "password": "peopleshistory",
                "roles": ["Contributor"],
            },
        )
        transaction.commit()

        self.assertEqual(201, response.status_code)
        howard = api.user.get(userid="howard")
        self.assertEqual("howard.zinn@example.com", howard.getProperty("email"))
        self.assertIn("Contributor", api.user.get_roles(username="howard"))

    def test_add_user_username_is_required(self):
        response = self.api_session.post("/@users", json={"password": "noamchomsky"})
        transaction.commit()

        self.assertEqual(400, response.status_code)
        self.assertTrue("Property 'username' is required" in response.text)

    def test_add_user_password_is_required(self):
        response = self.api_session.post("/@users", json={"username": "noamchomsky"})
        transaction.commit()

        self.assertEqual(400, response.status_code)
        self.assertTrue(
            ("You have to either send a " "password or sendPasswordReset")
            in response.text
        )

    def test_add_user_email_is_required_if_email_login_is_enabled(self):
        # enable use_email_as_login
        security_settings = getAdapter(self.portal, ISecuritySchema)
        security_settings.use_email_as_login = True
        transaction.commit()
        response = self.api_session.post(
            "/@users", json={"username": "noam", "password": "secret"}
        )

        self.assertEqual(400, response.status_code)
        self.assertTrue("Property 'username' is not allowed" in response.text)

    def test_add_user_email_with_email_login_enabled(self):
        # enable use_email_as_login
        security_settings = getAdapter(self.portal, ISecuritySchema)
        security_settings.use_email_as_login = True
        transaction.commit()
        response = self.api_session.post(
            "/@users", json={"email": "howard.zinn@example.com", "password": "secret"}
        )
        transaction.commit()

        self.assertEqual(201, response.status_code)
        self.assertTrue(api.user.get(userid="howard.zinn@example.com"))

    def test_username_is_not_allowed_with_email_login_enabled(self):
        # enable use_email_as_login
        security_settings = getAdapter(self.portal, ISecuritySchema)
        security_settings.use_email_as_login = True
        transaction.commit()
        response = self.api_session.post(
            "/@users",
            json={
                "username": "howard",
                "email": "howard.zinn@example.com",
                "password": "secret",
            },
        )
        transaction.commit()

        self.assertEqual(400, response.status_code)
        self.assertTrue("Property 'username' is not allowed" in response.text)

    def test_add_user_with_email_login_enabled(self):
        # enable use_email_as_login
        security_settings = getAdapter(self.portal, ISecuritySchema)
        security_settings.use_email_as_login = True
        transaction.commit()
        response = self.api_session.post(
            "/@users", json={"email": "howard.zinn@example.com", "password": "secret"}
        )
        transaction.commit()

        self.assertEqual(201, response.status_code)
        user = api.user.get(userid="howard.zinn@example.com")
        self.assertTrue(user)
        self.assertEqual("howard.zinn@example.com", user.getUserName())
        self.assertEqual("howard.zinn@example.com", user.getProperty("email"))

    def test_add_user_with_sendPasswordRest_sends_email(self):
        response = self.api_session.post(
            "/@users",
            json={
                "username": "howard",
                "email": "howard.zinn@example.com",
                "sendPasswordReset": True,
            },
        )
        transaction.commit()

        self.assertEqual(201, response.status_code)
        self.assertTrue("To: howard.zinn@example.com" in self.mailhost.messages[0])

    def test_add_user_send_properties(self):
        response = self.api_session.post(
            "/@users",
            json={
                "username": "howard",
                "password": "secret",
                "email": "howard.zinn@example.com",
                "fullname": "Howard Zinn",
            },
        )
        transaction.commit()

        self.assertEqual(201, response.status_code)
        member = api.user.get(username="howard")
        self.assertEqual(member.getProperty("fullname"), "Howard Zinn")

    def test_add_anon_user_sends_properties_are_saved(self):
        security_settings = getAdapter(self.portal, ISecuritySchema)
        security_settings.enable_self_reg = True
        transaction.commit()

        response = self.anon_api_session.post(
            "/@users",
            json={
                "username": "howard",
                "email": "howard.zinn@example.com",
                "fullname": "Howard Zinn",
            },
        )
        transaction.commit()

        self.assertEqual(201, response.status_code)
        member = api.user.get(username="howard")
        self.assertEqual(member.getProperty("fullname"), "Howard Zinn")

    def test_add_anon_no_roles(self):
        """Make sure anonymous users cannot set their own roles.
           Allowing so would make them Manager.
        """
        security_settings = getAdapter(self.portal, ISecuritySchema)
        security_settings.enable_self_reg = True
        transaction.commit()

        response = self.anon_api_session.post(
            "/@users",
            json={
                "username": "howard",
                "email": "howard.zinn@example.com",
                "roles": ["Manager"],
            },
        )
        transaction.commit()

        self.assertEqual(400, response.status_code)
        errors = response.json()["error"]["errors"]
        fields = [x["field"] for x in errors]
        self.assertEqual(["roles"], fields)

    def test_add_user_with_uuid_as_userid_enabled(self):
        # enable use_email_as_login
        security_settings = getAdapter(self.portal, ISecuritySchema)
        security_settings.use_email_as_login = True
        security_settings.use_uuid_as_userid = True
        transaction.commit()
        response = self.api_session.post(
            "/@users", json={"email": "howard.zinn@example.com", "password": "secret"}
        )
        transaction.commit()

        self.assertEqual(201, response.status_code)
        user_id = response.json()["id"]
        user = api.user.get(userid=user_id)
        self.assertTrue(user)
        self.assertEqual("howard.zinn@example.com", user.getUserName())
        self.assertEqual("howard.zinn@example.com", user.getProperty("email"))

    def test_get_user(self):
        response = self.api_session.get("/@users/noam")

        self.assertEqual(response.status_code, 200)
        self.assertEqual("noam", response.json().get("id"))
        self.assertEqual(
            self.portal.absolute_url() + "/@users/noam", response.json().get("@id")
        )
        self.assertEqual("noam.chomsky@example.com", response.json().get("email"))
        self.assertEqual("Noam Avram Chomsky", response.json().get("fullname"))
        self.assertEqual(
            "web.mit.edu/chomsky", response.json().get("home_page")
        )  # noqa
        self.assertEqual(
            "Professor of Linguistics", response.json().get("description")
        )  # noqa
        self.assertEqual("Cambridge, MA", response.json().get("location"))

    def test_get_user_as_anonymous(self):
        response = self.anon_api_session.get("/@users/noam")
        self.assertEqual(response.status_code, 401)

    def test_get_other_user_info_when_logged_in(self):
        noam_api_session = RelativeSession(self.portal_url)
        noam_api_session.headers.update({"Accept": "application/json"})
        noam_api_session.auth = ("noam", "password")

        response = noam_api_session.get("/@users/otheruser")
        self.assertEqual(response.status_code, 401)
        noam_api_session.close()

    def test_get_search_user_with_filter(self):
        response = self.api_session.post(
            "/@users",
            json={
                "username": "howard",
                "email": "howard.zinn@example.com",
                "password": "peopleshistory",
            },
        )
        transaction.commit()
        response = self.api_session.get("/@users", params={"query": "noa"})

        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.json()), 1)
        self.assertEqual("noam", response.json()[0].get("id"))
        self.assertEqual(
            self.portal.absolute_url() + "/@users/noam", response.json()[0].get("@id")
        )
        self.assertEqual("noam.chomsky@example.com", response.json()[0].get("email"))
        self.assertEqual(
            "Noam Avram Chomsky", response.json()[0].get("fullname")
        )  # noqa

        response = self.api_session.get("/@users", params={"query": "howa"})
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.json()), 1)
        self.assertEqual("howard", response.json()[0].get("id"))

    def test_get_search_user_with_filter_as_anonymous(self):
        response = self.api_session.post(
            "/@users",
            json={
                "username": "howard",
                "email": "howard.zinn@example.com",
                "password": "peopleshistory",
            },
        )
        transaction.commit()
        response = self.anon_api_session.get("/@users", params={"query": "howa"})
        self.assertEqual(response.status_code, 401)

    def test_get_search_user_with_filter_as_unauthorized_user(self):
        response = self.api_session.post(
            "/@users",
            json={
                "username": "howard",
                "email": "howard.zinn@example.com",
                "password": "peopleshistory",
            },
        )
        transaction.commit()
        noam_api_session = RelativeSession(self.portal_url)
        noam_api_session.headers.update({"Accept": "application/json"})
        noam_api_session.auth = ("noam", "password")

        response = noam_api_session.get("/@users", params={"query": "howa"})
        self.assertEqual(response.status_code, 401)
        noam_api_session.close()

    def test_get_non_existing_user(self):
        response = self.api_session.get("/@users/non-existing-user")

        self.assertEqual(response.status_code, 404)

    def test_update_user(self):
        payload = {
            "fullname": "Noam A. Chomsky",
            "username": "avram",
            "email": "avram.chomsky@example.com",
        }
        response = self.api_session.patch("/@users/noam", json=payload)
        transaction.commit()

        self.assertEqual(response.status_code, 204)
        noam = api.user.get(userid="noam")
        self.assertEqual("noam", noam.getUserId())  # user id never changes
        self.assertEqual("avram", noam.getUserName())
        self.assertEqual("Noam A. Chomsky", noam.getProperty("fullname"))
        self.assertEqual("avram.chomsky@example.com", noam.getProperty("email"))

    def test_user_can_update_himself(self):
        payload = {
            "fullname": "Noam A. Chomsky",
            "username": "noam",
            "email": "avram.chomsky@plone.org",
        }
        self.api_session.auth = ("noam", "password")
        response = self.api_session.patch("/@users/noam", json=payload)

        self.assertEqual(response.status_code, 204)
        transaction.commit()

        noam = api.user.get(userid="noam")
        self.assertEqual("noam", noam.getUserId())  # user id never changes
        self.assertEqual("Noam A. Chomsky", noam.getProperty("fullname"))
        self.assertEqual("avram.chomsky@plone.org", noam.getProperty("email"))

    def test_update_roles(self):
        self.assertNotIn("Contributor", api.user.get_roles(username="noam"))

        self.api_session.patch("/@users/noam", json={"roles": {"Contributor": True}})
        transaction.commit()
        self.assertIn("Contributor", api.user.get_roles(username="noam"))

        self.api_session.patch("/@users/noam", json={"roles": {"Contributor": False}})
        transaction.commit()
        self.assertNotIn("Contributor", api.user.get_roles(username="noam"))

    def test_update_user_password(self):
        old_password_hashes = dict(self.portal.acl_users.source_users._user_passwords)
        payload = {"password": "secret"}
        response = self.api_session.patch("/@users/noam", json=payload)
        transaction.commit()

        self.assertEqual(response.status_code, 204)

        new_password_hashes = dict(self.portal.acl_users.source_users._user_passwords)
        self.assertNotEqual(old_password_hashes["noam"], new_password_hashes["noam"])

    def test_update_portrait(self):
        payload = {
            "portrait": {
                "filename": "image.gif",
                "encoding": "base64",
                "data": u"R0lGODlhAQABAIAAAP///wAAACwAAAAAAQABAAACAkQBADs=",
                "content-type": "image/gif",
            }
        }
        self.api_session.auth = ("noam", "password")
        response = self.api_session.patch("/@users/noam", json=payload)

        self.assertEqual(response.status_code, 204)
        transaction.commit()

        user = self.api_session.get("/@users/noam").json()
        self.assertTrue(
            user.get("portrait").endswith("plone/portal_memberdata/portraits/noam")
        )

    def test_update_portrait_with_default_plone_scaling(self):
        payload = {
            "portrait": {
                "filename": "image.gif",
                "encoding": "base64",
                "data": u"R0lGODlhAQABAIAAAP///wAAACwAAAAAAQABAAACAkQBADs=",
                "content-type": "image/gif",
                "scale": True,
            }
        }
        self.api_session.auth = ("noam", "password")
        response = self.api_session.patch("/@users/noam", json=payload)

        self.assertEqual(response.status_code, 204)
        transaction.commit()

        user = self.api_session.get("/@users/noam").json()
        self.assertTrue(
            user.get("portrait").endswith("plone/portal_memberdata/portraits/noam")
        )

    def test_update_portrait_by_manager(self):
        payload = {
            "portrait": {
                "filename": "image.gif",
                "encoding": "base64",
                "data": u"R0lGODlhAQABAIAAAP///wAAACwAAAAAAQABAAACAkQBADs=",
                "content-type": "image/gif",
            }
        }
        response = self.api_session.patch("/@users/noam", json=payload)

        self.assertEqual(response.status_code, 204)
        transaction.commit()

        user = self.api_session.get("/@users/noam").json()
        self.assertTrue(
            user.get("portrait").endswith("plone/portal_memberdata/portraits/noam")
        )

    def test_delete_portrait(self):
        payload = {
            "portrait": None,
        }
        self.api_session.auth = ("noam", "password")
        response = self.api_session.patch("/@users/noam", json=payload)

        self.assertEqual(response.status_code, 204)
        transaction.commit()

        user = self.api_session.get("/@users/noam").json()

        self.assertTrue(user.get("portrait") is None)

    def test_delete_portrait_by_manager(self):
        payload = {
            "portrait": None,
        }
        response = self.api_session.patch("/@users/noam", json=payload)

        self.assertEqual(response.status_code, 204)
        transaction.commit()

        user = self.api_session.get("/@users/noam").json()

        self.assertTrue(user.get("portrait") is None)

    def test_update_user_with_portrait_set_without_updating_portrait(self):
        payload = {
            "portrait": {
                "filename": "image.gif",
                "encoding": "base64",
                "data": u"R0lGODlhAQABAIAAAP///wAAACwAAAAAAQABAAACAkQBADs=",
                "content-type": "image/gif",
            }
        }
        self.api_session.auth = ("noam", "password")
        response = self.api_session.patch("/@users/noam", json=payload)

        self.assertEqual(response.status_code, 204)
        transaction.commit()

        payload = {
            "fullname": "Noam A. Chomsky",
            "username": "noam",
            "email": "avram.chomsky@plone.org",
            "portrait": "http://localhost:55001/plone/portal_memberdata/portraits/noam",
        }
        self.api_session.auth = ("noam", "password")
        response = self.api_session.patch("/@users/noam", json=payload)

        self.assertEqual(response.status_code, 204)
        transaction.commit()

        user = self.api_session.get("/@users/noam").json()
        self.assertTrue(
            user.get("portrait").endswith("plone/portal_memberdata/portraits/noam")
        )

    def test_anonymous_user_can_not_update_existing_user(self):
        payload = {
            "fullname": "Noam A. Chomsky",
            "username": "noam",
            "email": "avram.chomsky@plone.org",
        }
        self.api_session.auth = ("noam", "password")
        response = self.anon_api_session.patch("/@users/noam", json=payload)

        self.assertEqual(response.status_code, 401)

    def test_user_can_not_update_another_user(self):
        payload = {
            "fullname": "Noam A. Chomsky",
            "username": "noam",
            "email": "avram.chomsky@plone.org",
        }
        self.api_session.auth = ("otheruser", "otherpassword")
        response = self.api_session.patch("/@users/noam", json=payload)

        self.assertEqual(response.status_code, 403)

    def test_user_requests_password_sends_password_via_mail(self):
        self.api_session.auth = ("noam", "password")
        payload = {}
        response = self.api_session.post("/@users/noam/reset-password", json=payload)
        transaction.commit()

        self.assertEqual(response.status_code, 200)
        # FIXME: Test that mail is sent

    def test_user_can_set_her_own_password(self):
        self.api_session.auth = ("noam", "password")
        self.portal.manage_permission(
            SetOwnPassword, roles=["Authenticated", "Manager"], acquire=False
        )
        transaction.commit()

        payload = {"old_password": "password", "new_password": "new_password"}
        response = self.api_session.post("/@users/noam/reset-password", json=payload)
        transaction.commit()

        self.assertEqual(response.status_code, 200)
        authed = self.portal.acl_users.authenticate("noam", "new_password", {})
        self.assertTrue(authed)

    def test_normal_authenticated_user_cannot_set_other_users_password(self):
        self.api_session.auth = ("noam", "password")
        self.portal.manage_permission(
            SetOwnPassword, roles=["Authenticated", "Manager"], acquire=False
        )
        transaction.commit()

        payload = {"old_password": "password", "new_password": "new_password"}
        response = self.api_session.post(
            "/@users/otheruser/reset-password", json=payload
        )
        transaction.commit()

        self.assertEqual(response.status_code, 403)
        self.assertEqual(response.json()["error"]["type"], "Wrong user")

    def test_user_set_own_password_requires_set_own_password_permission(self):
        self.api_session.auth = ("noam", "password")
        self.portal.manage_permission(SetOwnPassword, roles=["Manager"], acquire=False)
        transaction.commit()

        payload = {"old_password": "password", "new_password": "new_password"}
        response = self.api_session.post("/@users/noam/reset-password", json=payload)
        transaction.commit()

        self.assertEqual(response.status_code, 403)

    def test_user_set_own_password_requires_old_and_new_password(self):
        self.api_session.auth = ("noam", "password")
        payload = {"old_password": "password"}
        response = self.api_session.post("/@users/noam/reset-password", json=payload)

        self.assertEqual(response.status_code, 400)
        self.assertEqual(response.json()["error"]["type"], "Invalid parameters")
        payload = {"new_password": "new_password"}
        response = self.api_session.post("/@users/noam/reset-password", json=payload)

        self.assertEqual(response.status_code, 400)
        self.assertEqual(response.json()["error"]["type"], "Invalid parameters")

    def test_user_set_own_password_checks_old_password(self):
        self.api_session.auth = ("noam", "password")
        payload = {"new_password": "new_password", "old_password": "wrong_password"}
        response = self.api_session.post("/@users/noam/reset-password", json=payload)

        self.assertEqual(response.status_code, 403)
        self.assertEqual(response.json()["error"]["type"], "Wrong password")

    def test_user_set_reset_token_requires_new_password(self):
        self.api_session.auth = ("noam", "password")
        payload = {"reset_token": "abc"}
        response = self.api_session.post("/@users/noam/reset-password", json=payload)

        self.assertEqual(response.status_code, 400)
        self.assertEqual(response.json()["error"]["type"], "Invalid parameters")

    def test_reset_with_token(self):
        reset_tool = getToolByName(self.portal, "portal_password_reset")
        reset_info = reset_tool.requestReset("noam")
        token = reset_info["randomstring"]
        transaction.commit()

        payload = {"reset_token": token, "new_password": "new_password"}
        response = self.api_session.post("/@users/noam/reset-password", json=payload)
        transaction.commit()

        self.assertEqual(response.status_code, 200)
        authed = self.portal.acl_users.authenticate("noam", "new_password", {})
        self.assertTrue(authed)

    def test_reset_with_uuid_as_userid_and_login_email_using_id(self):
        # enable use_email_as_login
        security_settings = getAdapter(self.portal, ISecuritySchema)
        security_settings.use_email_as_login = True
        security_settings.use_uuid_as_userid = True
        transaction.commit()

        response = self.api_session.post(
            "/@users", json={"email": "howard.zinn@example.com", "password": "secret"}
        )
        transaction.commit()

        self.assertEqual(201, response.status_code)
        user_id = response.json()["id"]
        user = api.user.get(userid=user_id)
        self.assertTrue(user)

        reset_tool = getToolByName(self.portal, "portal_password_reset")
        reset_info = reset_tool.requestReset(user.id)
        token = reset_info["randomstring"]
        transaction.commit()

        payload = {"reset_token": token, "new_password": "new_password"}
        response = self.api_session.post(
            "/@users/{}/reset-password".format(user.id), json=payload
        )

        self.assertEqual(response.status_code, 200)

    def test_reset_with_uuid_as_userid_and_login_email_using_mail(self):
        # enable use_email_as_login
        security_settings = getAdapter(self.portal, ISecuritySchema)
        security_settings.use_email_as_login = True
        security_settings.use_uuid_as_userid = True
        transaction.commit()

        response = self.api_session.post(
            "/@users", json={"email": "howard.zinn@example.com", "password": "secret"}
        )
        transaction.commit()

        self.assertEqual(201, response.status_code)
        user_id = response.json()["id"]
        user = api.user.get(userid=user_id)
        self.assertTrue(user)

        reset_tool = getToolByName(self.portal, "portal_password_reset")
        reset_info = reset_tool.requestReset(user.id)
        token = reset_info["randomstring"]
        transaction.commit()

        payload = {"reset_token": token, "new_password": "new_password"}
        response = self.api_session.post(
            "/@users/{}/reset-password".format(user.getUserName()), json=payload
        )

        self.assertEqual(response.status_code, 200)

    def test_reset_and_login_email_using_mail(self):
        # enable use_email_as_login
        security_settings = getAdapter(self.portal, ISecuritySchema)
        security_settings.use_email_as_login = True
        transaction.commit()

        response = self.api_session.post(
            "/@users", json={"email": "howard.zinn@example.com", "password": "secret"}
        )
        transaction.commit()

        self.assertEqual(201, response.status_code)
        user_id = response.json()["id"]
        user = api.user.get(userid=user_id)
        self.assertTrue(user)

        reset_tool = getToolByName(self.portal, "portal_password_reset")
        reset_info = reset_tool.requestReset(user.id)
        token = reset_info["randomstring"]
        transaction.commit()

        payload = {"reset_token": token, "new_password": "new_password"}
        response = self.api_session.post(
            "/@users/{}/reset-password".format(user.getUserName()), json=payload
        )

        self.assertEqual(response.status_code, 200)

    def test_delete_user(self):
        response = self.api_session.delete("/@users/noam")
        transaction.commit()

        self.assertEqual(response.status_code, 204)
        self.assertEqual(None, api.user.get(userid="noam"))

    def test_delete_non_existing_user(self):
        response = self.api_session.delete("/@users/non-existing-user")
        transaction.commit()

        self.assertEqual(response.status_code, 404)

    def test_anonymous_requires_enable_self_reg(self):
        security_settings = getAdapter(self.portal, ISecuritySchema)
        security_settings.enable_self_reg = False
        transaction.commit()

        response = self.anon_api_session.post(
            "/@users", json={"password": "noamchomsky"}
        )
        transaction.commit()

        self.assertEqual(403, response.status_code)

        security_settings.enable_self_reg = True
        transaction.commit()

        response = self.anon_api_session.post(
            "/@users",
            json={"username": "new_user", "email": "avram.chomsky@example.com"},
        )
        transaction.commit()

        self.assertEqual(201, response.status_code)

    def test_anonymous_without_enable_user_pwd_choice_sends_mail(self):
        security_settings = getAdapter(self.portal, ISecuritySchema)
        security_settings.enable_self_reg = True
        transaction.commit()

        response = self.anon_api_session.post(
            "/@users",
            json={"username": "new_user", "email": "avram.chomsky@example.com"},
        )
        transaction.commit()

        self.assertEqual(201, response.status_code)
        self.assertTrue("To: avram.chomsky@example.com" in self.mailhost.messages[0])

    def test_anonymous_can_set_password_with_enable_user_pwd_choice(self):
        security_settings = getAdapter(self.portal, ISecuritySchema)
        security_settings.enable_self_reg = True
        transaction.commit()

        response = self.anon_api_session.post(
            "/@users",
            json={
                "username": "new_user",
                "email": "avram.chomsky@example.com",
                "password": "secret",
            },
        )
        transaction.commit()

        self.assertEqual(400, response.status_code)
        self.assertTrue("Property 'password' is not allowed" in response.text)

        security_settings.enable_user_pwd_choice = True
        transaction.commit()

        response = self.anon_api_session.post(
            "/@users",
            json={
                "username": "new_user",
                "email": "avram.chomsky@example.com",
                "password": "secret",
            },
        )
        transaction.commit()

        self.assertEqual(201, response.status_code)

    def test_anonymous_with_enable_user_pwd_choice_doent_send_email(self):
        security_settings = getAdapter(self.portal, ISecuritySchema)
        security_settings.enable_self_reg = True
        security_settings.enable_user_pwd_choice = True
        transaction.commit()

        response = self.anon_api_session.post(
            "/@users",
            json={
                "username": "new_user",
                "email": "avram.chomsky@example.com",
                "password": "secret",
            },
        )
        transaction.commit()

        self.assertEqual(self.mailhost.messages, [])
        self.assertEqual(201, response.status_code)

    def test_anonymous_with_enable_user_sets_only_member_role(self):
        security_settings = getAdapter(self.portal, ISecuritySchema)
        security_settings.enable_self_reg = True
        security_settings.enable_user_pwd_choice = True
        transaction.commit()

        response = self.anon_api_session.post(
            "/@users",
            json={
                "username": "new_user",
                "email": "avram.chomsky@example.com",
                "password": "secret",
            },
        )

        response = response.json()
        self.assertIn("Member", response["roles"])
        self.assertEqual(1, len(response["roles"]))

    def test_add_user_no_roles_sets_member_as_sensible_default(self):
        response = self.api_session.post(
            "/@users",
            json={
                "username": "howard",
                "email": "howard.zinn@example.com",
                "password": "peopleshistory",
            },
        )
        transaction.commit()

        self.assertEqual(201, response.status_code)

        response = response.json()

        self.assertIn("Member", response["roles"])
        self.assertEqual(1, len(response["roles"]))



# -*- coding: utf-8 -*-
from plone.restapi.imaging import get_original_image_url
from plone.restapi.imaging import get_scales
from plone.restapi.interfaces import IFieldSerializer
from plone.restapi.serializer.converters import json_compatible
from Products.Archetypes.interfaces import IBaseObject
from Products.Archetypes.interfaces.field import IField
from Products.Archetypes.interfaces.field import IFileField
from Products.Archetypes.interfaces.field import IImageField
from Products.Archetypes.interfaces.field import IReferenceField
from Products.Archetypes.interfaces.field import ITextField
from Products.CMFCore.utils import getToolByName
from six.moves import map
from zope.component import adapter
from zope.interface import implementer
from zope.interface import Interface

import pkg_resources


# When we have plone.app.collection, we need a serializer for IQueryField.
# On Plone 5 it comes from p.a.collection.
# On Plone 4 we also have p.a.collection, but the field is in archetypes.querywidget.
try:
    pkg_resources.get_distribution("plone.app.collection")
except pkg_resources.DistributionNotFound:
    IQueryField = None
else:
    try:
        from plone.app.collection.field import IQueryField
    except ImportError:
        from archetypes.querywidget.interfaces import IQueryField

try:
    pkg_resources.get_distribution("plone.app.blob")
except pkg_resources.DistributionNotFound:
    HAS_BLOB = False
else:
    HAS_BLOB = True
    from plone.app.blob.interfaces import IBlobField
    from plone.app.blob.interfaces import IBlobImageField


@adapter(IField, IBaseObject, Interface)
@implementer(IFieldSerializer)
class DefaultFieldSerializer(object):
    def __init__(self, field, context, request):
        self.context = context
        self.request = request
        self.field = field

    def __call__(self):
        accessor = self.field.getAccessor(self.context)
        return json_compatible(accessor())


@adapter(IFileField, IBaseObject, Interface)
@implementer(IFieldSerializer)
class FileFieldSerializer(DefaultFieldSerializer):
    def __call__(self):
        url = "/".join(
            (self.context.absolute_url(), "@@download", self.field.getName())
        )
        result = {
            "filename": self.field.getFilename(self.context),
            "content-type": self.field.getContentType(self.context),
            "size": self.field.get_size(self.context),
            "download": url,
        }
        return json_compatible(result)


@adapter(ITextField, IBaseObject, Interface)
@implementer(IFieldSerializer)
class TextFieldSerializer(DefaultFieldSerializer):
    def __call__(self):
        mimetypes_registry = getToolByName(self.context, "mimetypes_registry")
        data = super(TextFieldSerializer, self).__call__()
        return {
            "content-type": json_compatible(mimetypes_registry(data)[2].normalized()),
            "data": data,
        }


@adapter(IImageField, IBaseObject, Interface)
@implementer(IFieldSerializer)
class ImageFieldSerializer(DefaultFieldSerializer):
    def __call__(self):
        image = self.field.get(self.context)
        if not image:
            return None

        width, height = image.width, image.height
        url = get_original_image_url(self.context, self.field.__name__, width, height)

        scales = get_scales(self.context, self.field, width, height)
        result = {
            "filename": self.field.getFilename(self.context),
            "content-type": self.field.get(self.context).getContentType(),
            "size": self.field.get(self.context).get_size(),
            "download": url,
            "width": width,
            "height": height,
            "scales": scales,
        }
        return json_compatible(result)


if HAS_BLOB:

    @adapter(IBlobField, IBaseObject, Interface)
    @implementer(IFieldSerializer)
    class BlobFieldSerializer(FileFieldSerializer):
        pass

    @adapter(IBlobImageField, IBaseObject, Interface)
    @implementer(IFieldSerializer)
    class BlobImageFieldSerializer(ImageFieldSerializer):
        pass


@adapter(IReferenceField, IBaseObject, Interface)
@implementer(IFieldSerializer)
class ReferenceFieldSerializer(DefaultFieldSerializer):
    def __call__(self):
        accessor = self.field.getAccessor(self.context)
        refs = accessor()
        if self.field.multiValued:
            return [json_compatible(r.absolute_url()) for r in refs]
        else:
            if refs is None:
                return None
            return json_compatible(refs.absolute_url())


if IQueryField is not None:

    @adapter(IQueryField, IBaseObject, Interface)
    @implementer(IFieldSerializer)
    class QueryFieldSerializer(DefaultFieldSerializer):
        def __call__(self):
            raw_value = self.field.getRaw(self.context)
            return json_compatible(list(map(dict, raw_value)))



# -*- coding: utf-8 -*-
from plone.app.layout.navigation.navtree import buildFolderTree
from plone.app.layout.navigation.root import getNavigationRoot
from plone.restapi.interfaces import IExpandableElement
from plone.restapi.services import Service
from Products.CMFPlone.browser.navtree import NavtreeQueryBuilder
from Products.CMFPlone.browser.navtree import SitemapNavtreeStrategy
from zope.component import adapter
from zope.component import getMultiAdapter
from zope.component.hooks import getSite
from zope.interface import implementer
from zope.interface import Interface


class NavigationTreeQueryBuilder(NavtreeQueryBuilder):
    """Build a folder tree query
    """

    def __init__(self, context, depth):
        NavtreeQueryBuilder.__init__(self, context)
        self.query["path"] = {
            "query": "/".join(context.getPhysicalPath()),
            "navtree_start": 1,
            "depth": depth - 1,
        }


class CustomNavtreeStrategy(SitemapNavtreeStrategy):
    """The navtree strategy used for the default navigation portlet
    """

    def __init__(self, context):
        SitemapNavtreeStrategy.__init__(self, context, None)
        self.context = context
        self.bottomLevel = 0
        self.rootPath = self.getRootPath()

    def subtreeFilter(self, node):
        sitemapDecision = SitemapNavtreeStrategy.subtreeFilter(self, node)
        if sitemapDecision is False:
            return False
        depth = node.get("depth", 0)
        if depth > 0 and self.bottomLevel > 0 and depth >= self.bottomLevel:
            return False
        else:
            return True

    def getRootPath(self, topLevel=1):
        rootPath = getNavigationRoot(self.context)

        contextPath = "/".join(self.context.getPhysicalPath())
        if not contextPath.startswith(rootPath):
            return None
        contextSubPathElements = contextPath[len(rootPath) + 1 :]
        if contextSubPathElements:
            contextSubPathElements = contextSubPathElements.split("/")
            if len(contextSubPathElements) < topLevel:
                return None
            rootPath = (
                rootPath + "/" + "/".join(contextSubPathElements[:topLevel])
            )  # noqa
        else:
            return None

        return rootPath


@implementer(IExpandableElement)
@adapter(Interface, Interface)
class Navigation(object):
    def __init__(self, context, request):
        self.context = context
        self.request = request
        self.portal = getSite()

    def __call__(self, expand=False):
        if self.request.form.get("expand.navigation.depth", False):
            self.depth = int(self.request.form["expand.navigation.depth"])
        else:
            self.depth = 1

        result = {
            "navigation": {"@id": "{}/@navigation".format(self.context.absolute_url())}
        }
        if not expand:
            return result

        tabs = getMultiAdapter((self.context, self.request), name="portal_tabs_view")
        items = []
        for tab in tabs.topLevelTabs():
            if self.depth > 1:
                subitems = self.getTabSubTree(
                    tabUrl=tab["url"], tabPath=tab.get("path")
                )
                items.append(
                    {
                        "title": tab.get("title", tab.get("name")),
                        "@id": tab["url"] + "",
                        "description": tab.get("description", ""),
                        "items": subitems,
                    }
                )
            else:
                items.append(
                    {
                        "title": tab.get("title", tab.get("name")),
                        "@id": tab["url"] + "",
                        "description": tab.get("description", ""),
                    }
                )
        result["navigation"]["items"] = items
        return result

    def getTabSubTree(self, tabUrl="", tabPath=None):
        if tabPath is None:
            # get path for current tab's object
            tabPath = tabUrl.split(self.portal.absolute_url())[-1]

            if tabPath == "" or "/view" in tabPath:
                return ""

            if tabPath.startswith("/"):
                tabPath = tabPath[1:]
            elif tabPath.endswith("/"):
                # we need a real path, without a slash that might appear
                # at the end of the path occasionally
                tabPath = str(tabPath.split("/")[0])

            if "%20" in tabPath:
                # we have the space in object's ID that has to be
                # converted to the real spaces
                tabPath = tabPath.replace("%20", " ").strip()

        tabObj = self.portal.restrictedTraverse(tabPath, None)
        if tabObj is None:
            return ""

        strategy = CustomNavtreeStrategy(tabObj)
        queryBuilder = NavigationTreeQueryBuilder(tabObj, self.depth)
        query = queryBuilder()
        data = buildFolderTree(tabObj, obj=tabObj, query=query, strategy=strategy)

        return self.recurse(children=data.get("children", []), level=1)

    def recurse(self, children=None, level=0, bottomLevel=0):
        li = []
        for node in children:
            item = {"title": node["Title"], "description": node["Description"]}
            item["@id"] = node["getURL"]
            if bottomLevel <= 0 or level <= bottomLevel:
                nc = node["children"]
                nc = self.recurse(nc, level + 1, bottomLevel)
                if nc:
                    item["items"] = nc
            li.append(item)
        return li


class NavigationGet(Service):
    def reply(self):
        navigation = Navigation(self.context, self.request)
        return navigation(expand=True)["navigation"]



# -*- coding: utf-8 -*-
from plone import api
from plone.browserlayer.utils import registered_layers
from plone.restapi import PROJECT_NAME
from plone.restapi.testing import PLONE_RESTAPI_DX_INTEGRATION_TESTING
from Products.CMFCore.utils import getToolByName

import unittest


try:
    from Products.CMFPlone.utils import get_installer
except ImportError:  # Plone < 5.1
    HAS_INSTALLER = False
else:
    HAS_INSTALLER = True


class TestInstall(unittest.TestCase):

    layer = PLONE_RESTAPI_DX_INTEGRATION_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]

    def test_product_is_installed(self):
        """ Validate that our products GS profile has been run and the product
            installed
        """
        if HAS_INSTALLER:
            qi = get_installer(self.portal)
            installed = qi.is_product_installed(PROJECT_NAME)
        else:
            qi_tool = getToolByName(self.portal, "portal_quickinstaller")
            installed = PROJECT_NAME in [
                p["id"] for p in qi_tool.listInstalledProducts()
            ]
        self.assertTrue(installed, "package appears not to have been installed")


class TestUninstall(unittest.TestCase):

    layer = PLONE_RESTAPI_DX_INTEGRATION_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]

        if HAS_INSTALLER:
            qi = get_installer(self.portal)
            qi.uninstall_product(PROJECT_NAME)
            self.installed = qi.is_product_installed(PROJECT_NAME)
        else:
            qi_tool = getToolByName(self.portal, "portal_quickinstaller")
            with api.env.adopt_roles(["Manager"]):
                qi_tool.uninstallProducts(products=[PROJECT_NAME])
            self.installed = qi_tool.isProductInstalled(PROJECT_NAME)

    def test_uninstalled(self):
        self.assertFalse(self.installed)

    def test_addon_layer_removed(self):
        layers = [l.getName() for l in registered_layers()]
        self.assertNotIn("IPloneRestapiLayer", layers)



# -*- coding: utf-8 -*-
from plone.restapi.interfaces import ISerializeToJson
from plone.restapi.services import Service
from zope.component import ComponentLookupError
from zope.component import getMultiAdapter
from zope.component import getUtilitiesFor
from zope.component import getUtility
from zope.interface import implementer
from zope.publisher.interfaces import IPublishTraverse
from zope.schema.interfaces import IVocabularyFactory


@implementer(IPublishTraverse)
class VocabulariesGet(Service):
    def __init__(self, context, request):
        super(VocabulariesGet, self).__init__(context, request)
        self.params = []

    def publishTraverse(self, request, name):
        # Treat any path segments after /@vocabularies as parameters
        self.params.append(name)
        return self

    def _error(self, status, type, message):
        self.request.response.setStatus(status)
        return {"error": {"type": type, "message": message}}

    def reply(self):
        if len(self.params) == 0:
            return [
                {
                    "@id": "{}/@vocabularies/{}".format(
                        self.context.absolute_url(), vocab[0]
                    ),
                    "title": vocab[0],
                }
                for vocab in getUtilitiesFor(IVocabularyFactory)
            ]

        name = self.params[0]
        try:
            factory = getUtility(IVocabularyFactory, name=name)
        except ComponentLookupError:
            return self._error(
                404, "Not Found", "The vocabulary '{}' does not exist".format(name)
            )

        vocabulary = factory(self.context)
        vocabulary_name = self.params[0]
        serializer = getMultiAdapter(
            (vocabulary, self.request), interface=ISerializeToJson
        )
        return serializer(
            "{}/@vocabularies/{}".format(self.context.absolute_url(), vocabulary_name)
        )



# -*- coding: utf-8 -*-
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.restapi.testing import PLONE_RESTAPI_DX_FUNCTIONAL_TESTING
from plone.restapi.testing import RelativeSession

import transaction
import unittest


class TestServicesTypes(unittest.TestCase):

    layer = PLONE_RESTAPI_DX_FUNCTIONAL_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)

    def tearDown(self):
        self.api_session.close()

    def test_get_types(self):
        response = self.api_session.get("{}/@types".format(self.portal.absolute_url()))

        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            response.headers.get("Content-Type"),
            "application/json",
            "Sending a GET request to @types endpoint should respond with "
            + 'Content-Type: "application/json", not '
            + '"{}"'.format(response.headers.get("Content-Type")),
        )
        for item in response.json():
            self.assertEqual(sorted(item), sorted(["@id", "title", "addable"]))

    def test_get_types_document(self):
        response = self.api_session.get(
            "{}/@types/Document".format(self.portal.absolute_url())
        )

        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            response.headers.get("Content-Type"),
            "application/json+schema",
            "Sending a GET request to @types endpoint should respond with "
            + 'Content-Type: "application/json+schema", not '
            + '"{}"'.format(response.headers.get("Content-Type")),
        )

    def test_get_types_with_unknown_type(self):
        response = self.api_session.get(
            "{}/@types/UnknownType".format(self.portal.absolute_url())
        )

        self.assertEqual(response.status_code, 404)
        self.assertEqual(
            "application/json",
            response.headers.get("Content-Type"),
            "Sending a GET request to @types endpoint should respond with "
            + 'Content-Type: "application/json", not '
            + '"{}"'.format(response.headers.get("Content-Type")),
        )

    def test_types_endpoint_only_accessible_for_authenticated_users(self):
        self.api_session.auth = ()
        response = self.api_session.get("{}/@types".format(self.portal.absolute_url()))
        self.assertEqual(response.status_code, 401)

    def test_contextaware_addable(self):
        response = self.api_session.get("{}/@types".format(self.portal.absolute_url()))

        allowed_ids = [x.getId() for x in self.portal.allowedContentTypes()]

        response_allowed_ids = [
            x["@id"].split("/")[-1] for x in response.json() if x["addable"]
        ]

        # We check subset here, because only DX types are returned by the
        # endpoint.
        self.assertNotEqual([], allowed_ids)
        self.assertTrue(set(response_allowed_ids).issubset(set(allowed_ids)))

    def test_image_type(self):
        response = self.api_session.get("/@types/Image")
        response = response.json()
        self.assertIn("fieldsets", response)
        self.assertIn("image.data", response["properties"]["image"]["properties"])

    def test_file_type(self):
        response = self.api_session.get("/@types/File")
        response = response.json()
        self.assertIn("fieldsets", response)
        self.assertIn("file.data", response["properties"]["file"]["properties"])

    def test_event_type(self):
        response = self.api_session.get("/@types/Event")
        response = response.json()
        self.assertIn("title", response["properties"]["start"])

    def test_addable_types_for_non_manager_user(self):
        user = api.user.create(
            email="noam.chomsky@example.com", username="noam", password="12345"
        )

        folder = api.content.create(
            container=self.portal, id="folder", type="Folder", title=u"folder"
        )

        folder_cant_add = api.content.create(
            container=self.portal,
            id="folder_cant_add",
            type="Folder",
            title=u"folder_cant_add",
        )

        api.user.grant_roles(user=user, obj=folder, roles=["Contributor"])

        api.user.grant_roles(user=user, obj=folder_cant_add, roles=["Reader"])

        transaction.commit()

        self.api_session.auth = ("noam", "12345")
        # In the folder, the user should be able to add types since we granted
        # Contributor role on it
        response = self.api_session.get("/folder/@types")
        response = response.json()

        # Any addable type will do.
        self.assertTrue(any(a["addable"] for a in response))

        # In the folder where the user only have Reader role, no types are
        # addable
        response = self.api_session.get("/folder_cant_add/@types")
        response = response.json()

        self.assertEqual(len([a for a in response if a["addable"]]), 0)

        # and in the root Plone site there's no addable types
        response = self.api_session.get("/@types")
        response = response.json()

        self.assertEqual(len([a for a in response if a["addable"]]), 0)


class TestServicesTypesTranslatedTitles(unittest.TestCase):

    layer = PLONE_RESTAPI_DX_FUNCTIONAL_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.headers.update({"Accept-Language": "es"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)

        transaction.commit()

    def tearDown(self):
        self.api_session.close()

    def test_get_types_translated(self):
        response = self.api_session.get("{}/@types".format(self.portal.absolute_url()))

        self.assertEqual(response.status_code, 200)

        self.assertEqual(
            {
                u"Archivo",
                u"Carpeta",
                u"Colección",
                u"DX Test Document",
                u"Enlace",
                u"Evento",
                u"Imagen",
                u"Noticia",
                u"Página",
            },
            set(item["title"] for item in response.json()),
        )



# -*- coding: utf-8 -*-
from plone.restapi.deserializer import json_body
from plone.restapi.deserializer.mixins import OrderingMixin
from plone.restapi.interfaces import IDeserializeFromJson
from Products.CMFPlone.interfaces import IPloneSiteRoot
from zope.component import adapter
from zope.interface import implementer
from zope.publisher.interfaces import IRequest

import json


@implementer(IDeserializeFromJson)
@adapter(IPloneSiteRoot, IRequest)
class DeserializeSiteRootFromJson(OrderingMixin, object):
    """JSON deserializer for the Plone site root
    """

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def __call__(self, validate_all=False):
        # Currently we only do layout and ordering, as the plone site root
        # has no schema or something like that.
        data = json_body(self.request)

        if "layout" in data:
            layout = data["layout"]
            self.context.setLayout(layout)

        # OrderingMixin
        if "ordering" in data and "subset_ids" not in data["ordering"]:
            data["ordering"]["subset_ids"] = self.context.contentIds()
        self.handle_ordering(data)

        # Volto Blocks on the Plone Site root faker
        if "blocks" in data:
            if not getattr(self.context, "blocks", False):
                self.context.manage_addProperty(
                    "blocks", json.dumps(data["blocks"]), "string"
                )  # noqa
            else:
                self.context.manage_changeProperties(
                    blocks=json.dumps(data["blocks"])
                )  # noqa

        if "blocks_layout" in data:
            if not getattr(self.context, "blocks_layout", False):
                self.context.manage_addProperty(
                    "blocks_layout", json.dumps(data["blocks_layout"]), "string"
                )  # noqa
            else:
                self.context.manage_changeProperties(
                    blocks_layout=json.dumps(data["blocks_layout"])
                )  # noqa

        if "title" in data:
            self.context.setTitle(data["title"])

        if "description" in data:
            self.context.manage_changeProperties(
                description=data["description"]
            )  # noqa

        return self.context



# -*- coding: utf-8 -*-
from DateTime import DateTime
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.restapi.interfaces import IExpandableElement
from plone.restapi.interfaces import ISerializeToJson
from plone.restapi.testing import HAS_AT
from plone.restapi.testing import PLONE_RESTAPI_AT_INTEGRATION_TESTING
from plone.restapi.tests.test_expansion import ExpandableElementFoo
from zope.component import getGlobalSiteManager
from zope.component import getMultiAdapter
from zope.component import provideAdapter
from zope.interface import Interface
from zope.publisher.interfaces.browser import IBrowserRequest

import json
import unittest


class TestATContentSerializer(unittest.TestCase):

    layer = PLONE_RESTAPI_AT_INTEGRATION_TESTING

    def setUp(self):
        if not HAS_AT:
            raise unittest.SkipTest("Skip tests if Archetypes is not present")
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        setRoles(self.portal, TEST_USER_ID, ["Contributor"])

        self.doc1 = self.portal[
            self.portal.invokeFactory(
                "ATTestDocument", id="doc1", title="Test Document"
            )
        ]

        self.doc1.setCreationDate(DateTime("2016-01-21T01:14:48+00:00"))
        self.doc1.setModificationDate(DateTime("2016-01-21T01:24:11+00:00"))
        self.doc1._setUID("76644b6611ab44c6881efd9cb17db12e")

    def serialize(self, obj):
        serializer = getMultiAdapter((obj, self.request), ISerializeToJson)
        return serializer()

    def test_serializer_returns_json_serializeable_object(self):
        obj = self.serialize(self.doc1)
        self.assertTrue(isinstance(json.dumps(obj), str), "Not JSON serializable")

    @unittest.skip("We do not include the context at this point")
    def test_serializer_includes_context(self):
        obj = self.serialize(self.doc1)
        self.assertIn(u"@context", obj)
        self.assertEqual(u"http://www.w3.org/ns/hydra/context.jsonld", obj[u"@context"])

    def test_serializer_includes_json_ld_id(self):
        obj = self.serialize(self.doc1)
        self.assertIn(u"@id", obj)
        self.assertEqual(self.doc1.absolute_url(), obj[u"@id"])

    def test_serializer_includes_id(self):
        obj = self.serialize(self.doc1)
        self.assertIn(u"id", obj)
        self.assertEqual(self.doc1.id, obj[u"id"])

    def test_serializer_includes_type(self):
        obj = self.serialize(self.doc1)
        self.assertIn(u"@type", obj)
        self.assertEqual(self.doc1.portal_type, obj[u"@type"])

    def test_serializer_includes_review_state(self):
        obj = self.serialize(self.doc1)
        self.assertIn(u"review_state", obj)
        self.assertEqual(u"private", obj[u"review_state"])

    def test_serializer_includes_uid(self):
        obj = self.serialize(self.doc1)
        self.assertIn(u"UID", obj)
        self.assertEqual(u"76644b6611ab44c6881efd9cb17db12e", obj[u"UID"])

    def test_serializer_includes_creation_date(self):
        obj = self.serialize(self.doc1)
        self.assertIn(u"creation_date", obj)
        self.assertEqual(u"2016-01-21T01:14:48+00:00", obj[u"creation_date"])

    def test_serializer_includes_modification_date(self):
        obj = self.serialize(self.doc1)
        self.assertIn(u"modification_date", obj)
        self.assertEqual(u"2016-01-21T01:24:11+00:00", obj[u"modification_date"])

    def test_serializer_ignores_field_without_read_permission(self):
        self.doc1.setTestReadPermissionField(u"Secret Stuff")
        setRoles(self.portal, TEST_USER_ID, ["Member"])
        self.assertNotIn(u"testReadPermissionField", self.serialize(self.doc1))

    def test_serializer_ignores_writeonly_field(self):
        self.doc1.setTestWriteonlyField(u"Secret Stuff")
        setRoles(self.portal, TEST_USER_ID, ["Member"])
        self.assertNotIn(u"testWriteonlyField", self.serialize(self.doc1))

    def test_serializer_includes_folder_items(self):
        folder = self.portal[
            self.portal.invokeFactory("ATTestFolder", id="folder", title="Test Folder")
        ]
        folder.invokeFactory("ATTestFolder", id="subfolder", title="Subfolder")
        folder.invokeFactory("ATTestDocument", id="doc", title="A Document")
        obj = self.serialize(folder)
        self.assertIn("items", obj)
        self.assertDictEqual(
            {
                "@id": "http://nohost/plone/folder/subfolder",
                "@type": "ATTestFolder",
                "description": "",
                "title": u"Subfolder",
                "review_state": "private",
            },
            obj["items"][0],
        )
        self.assertDictEqual(
            {
                "@id": "http://nohost/plone/folder/doc",
                "@type": "ATTestDocument",
                "description": "",
                "title": u"A Document",
                "review_state": "private",
            },
            obj["items"][1],
        )

    def test_serializer_orders_folder_items_by_get_object_position_in_parent(
        self,
    ):  # noqa
        folder = self.portal[
            self.portal.invokeFactory("ATTestFolder", id="folder", title="Test Folder")
        ]
        folder.invokeFactory("ATTestDocument", id="doc1", title="A Document")
        folder.invokeFactory("ATTestDocument", id="doc2", title="Second doc")

        # Change GOPIP (getObjectPositionInParent) based order
        folder.moveObjectsUp("doc2")

        obj = self.serialize(folder)

        self.assertIn("items", obj)
        self.assertEqual(
            obj["items"],
            [
                {
                    "@id": "http://nohost/plone/folder/doc2",
                    "@type": "ATTestDocument",
                    "description": "",
                    "title": u"Second doc",
                    "review_state": "private",
                },
                {
                    "@id": "http://nohost/plone/folder/doc1",
                    "@type": "ATTestDocument",
                    "description": "",
                    "title": u"A Document",
                    "review_state": "private",
                },
            ],
        )

    def test_get_layout(self):
        current_layout = self.doc1.getLayout()
        obj = self.serialize(self.doc1)
        self.assertIn("layout", obj)
        self.assertEqual(current_layout, obj["layout"])

    def test_serializer_includes_expansion(self):
        provideAdapter(
            ExpandableElementFoo,
            adapts=(Interface, IBrowserRequest),
            provides=IExpandableElement,
            name="foo",
        )
        obj = self.serialize(self.doc1)
        self.assertIn("foo", obj["@components"])
        self.assertEqual("collapsed", obj["@components"]["foo"])
        gsm = getGlobalSiteManager()
        gsm.unregisterAdapter(
            ExpandableElementFoo,
            (Interface, IBrowserRequest),
            IExpandableElement,
            "foo",
        )

    def test_get_is_folderish_in_folder(self):
        self.portal.invokeFactory("Folder", id=u"folder")
        serializer = getMultiAdapter(
            (self.portal.folder, self.request), ISerializeToJson
        )
        obj = serializer()
        self.assertIn("is_folderish", obj)
        self.assertEqual(True, obj["is_folderish"])

    def test_nextprev_no_nextprev(self):
        folder = api.content.create(
            container=self.portal,
            type="ATTestFolder",
            title="Folder with items",
            description="This is a folder with some documents",
        )
        doc = api.content.create(
            container=folder,
            type="ATTestDocument",
            title="Item 1",
            description="One item alone in the folder",
        )
        data = self.serialize(doc)
        self.assertEqual({}, data["previous_item"])
        self.assertEqual({}, data["next_item"])

    def test_nextprev_has_prev(self):
        folder = api.content.create(
            container=self.portal,
            type="ATTestFolder",
            title="Folder with items",
            description="This is a folder with some documents",
        )
        api.content.create(
            container=folder,
            type="ATTestDocument",
            title="Item 1",
            description="Previous item",
        )
        doc = api.content.create(
            container=folder,
            type="ATTestDocument",
            title="Item 2",
            description="Current item",
        )
        data = self.serialize(doc)
        self.assertEqual(
            {
                "@id": "http://nohost/plone/folder-with-items/item-1",
                "@type": "ATTestDocument",
                "title": "Item 1",
                "description": "Previous item",
            },
            data["previous_item"],
        )
        self.assertEqual({}, data["next_item"])

    def test_nextprev_has_next(self):
        folder = api.content.create(
            container=self.portal,
            type="ATTestFolder",
            title="Folder with items",
            description="This is a folder with some documents",
        )
        doc = api.content.create(
            container=folder,
            type="ATTestDocument",
            title="Item 1",
            description="Current item",
        )
        api.content.create(
            container=folder,
            type="ATTestDocument",
            title="Item 2",
            description="Next item",
        )
        data = self.serialize(doc)
        self.assertEqual({}, data["previous_item"])
        self.assertEqual(
            {
                "@id": "http://nohost/plone/folder-with-items/item-2",
                "@type": "ATTestDocument",
                "title": "Item 2",
                "description": "Next item",
            },
            data["next_item"],
        )

    def test_nextprev_has_nextprev(self):
        folder = api.content.create(
            container=self.portal,
            type="ATTestFolder",
            title="Folder with items",
            description="This is a folder with some documents",
        )
        api.content.create(
            container=folder,
            type="ATTestDocument",
            title="Item 1",
            description="Previous item",
        )
        doc = api.content.create(
            container=folder,
            type="ATTestDocument",
            title="Item 2",
            description="Current item",
        )
        api.content.create(
            container=folder,
            type="ATTestDocument",
            title="Item 3",
            description="Next item",
        )
        data = self.serialize(doc)
        self.assertEqual(
            {
                "@id": "http://nohost/plone/folder-with-items/item-1",
                "@type": "ATTestDocument",
                "title": "Item 1",
                "description": "Previous item",
            },
            data["previous_item"],
        )
        self.assertEqual(
            {
                "@id": "http://nohost/plone/folder-with-items/item-3",
                "@type": "ATTestDocument",
                "title": "Item 3",
                "description": "Next item",
            },
            data["next_item"],
        )

    def test_nextprev_root_no_nextprev(self):
        data = self.serialize(self.doc1)
        self.assertEqual({}, data["previous_item"])
        self.assertEqual({}, data["next_item"])

    def test_nextprev_root_has_prev(self):
        doc = api.content.create(
            container=self.portal,
            type="ATTestDocument",
            title="Item 2",
            description="Current item",
        )
        data = self.serialize(doc)
        self.assertEqual(
            {
                "@id": "http://nohost/plone/doc1",
                "@type": "ATTestDocument",
                "title": "Test Document",
                "description": "",
            },
            data["previous_item"],
        )
        self.assertEqual({}, data["next_item"])

    def test_nextprev_root_has_next(self):
        api.content.create(
            container=self.portal,
            type="ATTestDocument",
            title="Item 2",
            description="Next item",
        )
        data = self.serialize(self.doc1)
        self.assertEqual({}, data["previous_item"])
        self.assertEqual(
            {
                "@id": "http://nohost/plone/item-2",
                "@type": "ATTestDocument",
                "title": "Item 2",
                "description": "Next item",
            },
            data["next_item"],
        )

    def test_nextprev_root_has_nextprev(self):
        api.content.create(
            container=self.portal,
            type="ATTestDocument",
            title="Item 1",
            description="Previous item",
        )
        doc = api.content.create(
            container=self.portal,
            type="ATTestDocument",
            title="Item 2",
            description="Current item",
        )
        api.content.create(
            container=self.portal,
            type="ATTestDocument",
            title="Item 3",
            description="Next item",
        )
        data = self.serialize(doc)
        self.assertEqual(
            {
                "@id": "http://nohost/plone/item-1",
                "@type": "ATTestDocument",
                "title": "Item 1",
                "description": "Previous item",
            },
            data["previous_item"],
        )
        self.assertEqual(
            {
                "@id": "http://nohost/plone/item-3",
                "@type": "ATTestDocument",
                "title": "Item 3",
                "description": "Next item",
            },
            data["next_item"],
        )



# -*- coding: utf-8 -*-
from plone.app.collection.interfaces import ICollection
from plone.restapi.batching import HypermediaBatch
from plone.restapi.interfaces import ISerializeToJson
from plone.restapi.interfaces import ISerializeToJsonSummary
from plone.restapi.serializer.atcontent import SerializeToJson
from zope.component import adapter
from zope.component import getMultiAdapter
from zope.interface import implementer
from zope.interface import Interface


@implementer(ISerializeToJson)
@adapter(ICollection, Interface)
class SerializeCollectionToJson(SerializeToJson):
    def __call__(self, version=None, include_items=True):
        collection_metadata = super(SerializeCollectionToJson, self).__call__(
            version=version
        )
        brains = self.context.results(batch=False)
        batch = HypermediaBatch(self.request, brains)

        results = collection_metadata
        if not self.request.form.get("fullobjects"):
            results["@id"] = batch.canonical_url
        results["items_total"] = batch.items_total
        if batch.links:
            results["batching"] = batch.links

        if "fullobjects" in list(self.request.form):
            results["items"] = [
                getMultiAdapter((brain.getObject(), self.request), ISerializeToJson)()
                for brain in batch
            ]
        else:
            results["items"] = [
                getMultiAdapter((brain, self.request), ISerializeToJsonSummary)()
                for brain in batch
            ]
        return results



# -*- coding: utf-8 -*-
from plone.app.contentlisting.interfaces import IContentListingObject
from plone.restapi.interfaces import ISerializeToJsonSummary
from plone.restapi.serializer.converters import json_compatible
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.WorkflowCore import WorkflowException
from Products.CMFPlone.interfaces import IPloneSiteRoot
from zope.component import adapter
from zope.interface import implementer
from zope.interface import Interface

# fmt: off
DEFAULT_METADATA_FIELDS = set([
    '@id',
    '@type',
    'description',
    'review_state',
    'title',
])

FIELD_ACCESSORS = {
    "@id": "getURL",
    "@type": "PortalType",
    "description": "Description",
    "title": "Title",
}

NON_METADATA_ATTRIBUTES = set([
    "getPath",
    "getURL",
])

BLACKLISTED_ATTRIBUTES = set([
    'getDataOrigin',
    'getObject',
    'getUserData',
])
# fmt: on


@implementer(ISerializeToJsonSummary)
@adapter(Interface, Interface)
class DefaultJSONSummarySerializer(object):
    """Default ISerializeToJsonSummary adapter.

    Requires context to be adaptable to IContentListingObject, which is
    the case for all content objects providing IContentish.
    """

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def __call__(self):
        obj = IContentListingObject(self.context)

        summary = {}
        for field in self.metadata_fields():
            if field.startswith("_") or field in BLACKLISTED_ATTRIBUTES:
                continue
            accessor = FIELD_ACCESSORS.get(field, field)
            value = getattr(obj, accessor, None)
            try:
                if callable(value):
                    value = value()
            except WorkflowException:
                summary[field] = None
                continue
            summary[field] = json_compatible(value)
        return summary

    def metadata_fields(self):
        additional_metadata_fields = self.request.form.get("metadata_fields", [])
        if not isinstance(additional_metadata_fields, list):
            additional_metadata_fields = [additional_metadata_fields]
        additional_metadata_fields = set(additional_metadata_fields)

        if "_all" in additional_metadata_fields:
            fields_cache = self.request.get("_summary_fields_cache", None)
            if fields_cache is None:
                catalog = getToolByName(self.context, "portal_catalog")
                fields_cache = set(catalog.schema()) | NON_METADATA_ATTRIBUTES
                self.request.set("_summary_fields_cache", fields_cache)
            additional_metadata_fields = fields_cache

        return DEFAULT_METADATA_FIELDS | additional_metadata_fields


@implementer(ISerializeToJsonSummary)
@adapter(IPloneSiteRoot, Interface)
class SiteRootJSONSummarySerializer(object):
    """ISerializeToJsonSummary adapter for the Plone Site root.
    """

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def __call__(self):
        summary = json_compatible(
            {
                "@id": self.context.absolute_url(),
                "@type": self.context.portal_type,
                "title": self.context.title,
                "description": self.context.description,
            }
        )
        return summary



# -*- coding: utf-8 -*-

# # Required to use the REST API at all, in addition to service specific
# permissions. Granted to Anonymous (i.e. everyone) by default via rolemap.xml

UseRESTAPI = "plone.restapi: Use REST API"



# -*- coding: utf-8 -*-
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import TEST_USER_PASSWORD
from plone.restapi.testing import PLONE_RESTAPI_DX_FUNCTIONAL_TESTING

import requests
import transaction
import unittest


class TestFunctionalAuth(unittest.TestCase):

    layer = PLONE_RESTAPI_DX_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        login(self.portal, SITE_OWNER_NAME)
        self.private_document = self.portal[
            self.portal.invokeFactory("Document", id="doc1", title="My Document")
        ]
        self.private_document_url = self.private_document.absolute_url()
        transaction.commit()

    def test_login_without_credentials_fails(self):
        response = requests.post(
            self.portal_url + "/@login", headers={"Accept": "application/json"}
        )
        self.assertEqual(400, response.status_code)
        self.assertEqual(
            u"Missing credentials", response.json().get("error").get("type")
        )
        self.assertEqual(
            u"Login and password must be provided in body.",
            response.json().get("error").get("message"),
        )

    def test_login_with_invalid_credentials_fails(self):
        response = requests.post(
            self.portal_url + "/@login",
            headers={"Accept": "application/json"},
            json={"login": "invalid", "password": "invalid"},
        )
        self.assertEqual(401, response.status_code)
        self.assertEqual(
            u"Invalid credentials", response.json().get("error").get("type")
        )
        self.assertEqual(
            u"Wrong login and/or password.", response.json().get("error").get("message")
        )

    def test_login_with_valid_credentials_returns_token(self):
        response = requests.post(
            self.portal_url + "/@login",
            headers={"Accept": "application/json"},
            json={"login": TEST_USER_NAME, "password": TEST_USER_PASSWORD},
        )
        self.assertEqual(200, response.status_code)
        self.assertTrue(u"token" in response.json())

    def test_accessing_private_document_with_valid_token_succeeds(self):
        # login and generate a valid token
        response = requests.post(
            self.portal_url + "/@login",
            headers={"Accept": "application/json"},
            json={"login": TEST_USER_NAME, "password": TEST_USER_PASSWORD},
        )
        valid_token = response.json().get("token")

        # use valid token to access a private resource
        response = requests.get(
            self.private_document_url,
            headers={
                "Accept": "application/json",
                "Authorization": "Bearer " + valid_token,
            },
        )

        self.assertEqual(200, response.status_code)
        self.assertTrue(u"@id" in response.json())

    def test_accessing_private_document_with_invalid_token_fails(self):
        invalid_token = "abcd1234"
        response = requests.get(
            self.private_document_url,
            headers={
                "Accept": "application/json",
                "Authorization": "Bearer " + invalid_token,
            },
        )

        self.assertEqual(401, response.status_code)
        self.assertEqual(u"Unauthorized", response.json().get("type"))
        self.assertEqual(
            u"You are not authorized to access this resource.",
            response.json().get("message"),
        )

    def test_accessing_private_document_with_expired_token_fails(self):
        # generate an expired token
        self.portal.acl_users.jwt_auth.store_tokens = True
        expired_token = self.portal.acl_users.jwt_auth.create_token(
            "admin", timeout=-60
        )
        transaction.commit()

        response = requests.get(
            self.private_document_url,
            headers={
                "Accept": "application/json",
                "Authorization": "Bearer " + expired_token,
            },
        )

        self.assertEqual(401, response.status_code)
        self.assertEqual(u"Unauthorized", response.json().get("type"))
        self.assertEqual(
            u"You are not authorized to access this resource.",
            response.json().get("message"),
        )



# -*- coding: utf-8 -*-
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.dexterity.interfaces import IDexterityItem
from plone.restapi.exceptions import DeserializationError
from plone.restapi.interfaces import IDeserializeFromJson
from plone.restapi.interfaces import ISerializeToJson
from plone.restapi.testing import PLONE_RESTAPI_DX_INTEGRATION_TESTING
from plone.restapi.tests.dxtypes import ITestAnnotationsBehavior
from plone.restapi.tests.mixin_ordering import OrderingMixin
from six.moves import range
from zExceptions import BadRequest
from zope.component import getMultiAdapter
from zope.component import provideHandler
from zope.lifecycleevent.interfaces import IObjectModifiedEvent

import json
import six
import unittest


class TestDXContentDeserializer(unittest.TestCase, OrderingMixin):

    layer = PLONE_RESTAPI_DX_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]

        self.portal.invokeFactory(
            "DXTestDocument",
            id=u"doc1",
            test_textline_field=u"Test Document",
            test_readonly_field=u"readonly",
        )

        # ordering setup
        self.folder = self.portal[
            self.portal.invokeFactory("Folder", id="folder1", title="Test folder")
        ]

        for x in range(1, 10):
            self.folder.invokeFactory(
                "Document", id="doc" + str(x), title="Test doc " + str(x)
            )

    def deserialize(self, body="{}", validate_all=False, context=None):
        context = context or self.portal.doc1
        self.request["BODY"] = body
        deserializer = getMultiAdapter((context, self.request), IDeserializeFromJson)
        return deserializer(validate_all=validate_all)

    def test_deserializer_raises_with_invalid_body(self):
        with self.assertRaises(DeserializationError) as cm:
            self.deserialize(body="Not a JSON object")
        self.assertEqual("No JSON object could be decoded", cm.exception.msg)

    def test_deserializer_raises_with_malformed_body(self):
        with self.assertRaises(DeserializationError) as cm:
            self.deserialize(body="[1,2,3]")
        self.assertEqual("Malformed body", cm.exception.msg)

    def test_deserializer_updates_field_value(self):
        self.deserialize(body='{"test_textline_field": "My Item"}')
        self.assertEqual(u"My Item", self.portal.doc1.test_textline_field)

    def test_deserializer_ignores_readonly_fields(self):
        self.deserialize(body='{"test_readonly_field": "Foo"}')
        self.assertEqual(u"readonly", self.portal.doc1.test_readonly_field)

    def test_deserializer_notifies_object_modified(self):
        def handler(obj, event):
            obj._handler_called = True

        provideHandler(handler, (IDexterityItem, IObjectModifiedEvent))
        self.deserialize(body='{"test_textline_field": "My Item"}')
        self.assertTrue(
            getattr(self.portal.doc1, "_handler_called", False),
            "IObjectEditedEvent not notified",
        )

    def test_deserializer_modified_event_contains_descriptions(self):
        def handler(obj, event):
            self.event = event

        provideHandler(handler, (IDexterityItem, IObjectModifiedEvent))
        self.deserialize(body='{"test_textline_field": "My Item"}')
        self.assertEqual(1, len(self.event.descriptions))
        self.assertEqual(
            ("IDXTestDocumentSchema.test_textline_field",),
            self.event.descriptions[0].attributes,
        )

    def test_deserializer_notifies_when_field_is_set_to_null(self):
        def handler(obj, event):
            obj._handler_called = True
            self.event = event

        provideHandler(handler, (IDexterityItem, IObjectModifiedEvent))
        self.deserialize(body='{"test_textline_field": null}')
        self.assertTrue(
            getattr(self.portal.doc1, "_handler_called", False),
            "IObjectModifiedEvent not notified",
        )
        self.assertEqual(
            ("IDXTestDocumentSchema.test_textline_field",),
            self.event.descriptions[0].attributes,
        )

    def test_deserializer_does_not_update_field_without_write_permission(self):
        self.portal.doc1.test_write_permission_field = u"Test Write Permission"
        setRoles(self.portal, TEST_USER_ID, ["Member", "Contributor", "Editor"])
        self.deserialize(body='{"test_write_permission_field": "Foo"}')
        self.assertEqual(
            u"Test Write Permission", self.portal.doc1.test_write_permission_field
        )

    def test_deserializer_updates_field_with_write_permission(self):
        self.portal.doc1.test_write_permission_field = u"Test Write Permission"
        setRoles(self.portal, TEST_USER_ID, ["Member", "Manager"])
        self.deserialize(body='{"test_write_permission_field": "Foo"}')
        self.assertEqual(u"Foo", self.portal.doc1.test_write_permission_field)

    def test_deserializer_validates_invariant(self):
        with self.assertRaises(BadRequest) as cm:
            self.deserialize(
                body='{"test_invariant_field1": "Foo",'
                ' "test_invariant_field2": "Bar"}'
            )
        self.assertEqual(
            u"Must have same values", str(cm.exception.args[0][0]["message"])
        )

    def test_deserializer_updates_behavior_field_value(self):
        self.deserialize(body='{"test_behavior_field": "My Value"}')
        self.assertEqual(u"My Value", self.portal.doc1.test_behavior_field)

    def test_deserializer_updates_behavior_field_value_in_annotations(self):
        self.deserialize(body='{"test_annotations_behavior_field": "My Value"}')
        self.assertEqual(
            u"My Value",
            ITestAnnotationsBehavior(self.portal.doc1).test_annotations_behavior_field,
        )

    def test_deserializer_raises_if_required_value_is_missing(self):
        # Value missing from request
        with self.assertRaises(BadRequest) as cm:
            self.deserialize(
                body='{"test_textline_field": "My Value"}', validate_all=True
            )
        self.assertEqual(
            u"Required input is missing.", cm.exception.args[0][0]["message"]
        )

        # An empty string should be considered a missing value
        with self.assertRaises(BadRequest) as cm:
            self.deserialize(body='{"test_textline_field": ""}', validate_all=True)
        self.assertEqual(
            u"Required input is missing.", cm.exception.args[0][0]["message"]
        )

    def test_deserializer_succeeds_if_required_value_is_provided(self):
        self.deserialize(body='{"test_required_field": "My Value"}', validate_all=True)
        self.assertEqual(u"My Value", self.portal.doc1.test_required_field)

    def test_deserializer_does_not_store_default_value(self):
        # XXX: Dexterity has an odd behavior with default values.
        # If a field's value is set to it's default value, it is not stored
        # because z3c.form only updates a field's value if the new value is
        # different from the previous one. Dexterity has a fallback to lookup
        # an attribute's value from the schema's default value if the attribute
        # doesn't exist. Thus the previous value is the default value and the
        # field doesn't get updated if the new value is also the default value.
        # Right now, we want to have the same behavior in the API for
        # consistency reasons.
        self.deserialize(body='{"test_default_value_field": "Default"}')
        self.assertNotIn(
            "test_default_value_field",
            dir(self.portal.doc1),
            "Default value unexpectedly stored.",
        )

    def test_deserializer_passes_validation_with_not_provided_defaults(self):
        self.deserialize(body='{"test_required_field": "My Value"}', validate_all=True)
        self.assertEqual(u"Default", self.portal.doc1.test_default_value_field)
        self.assertEqual(u"DefaultFactory", self.portal.doc1.test_default_factory_field)

    def test_deserializer_sets_missing_value_when_receiving_null(self):
        self.deserialize(body='{"test_missing_value_field": null}')
        self.assertEqual(u"missing", self.portal.doc1.test_missing_value_field)

    def test_deserializer_sets_missing_value_on_required_field(self):
        """We don't set missing_value if the field is required"""
        self.deserialize(body='{"test_missing_value_required_field": "valid value"}')
        with self.assertRaises(BadRequest) as cm:
            self.deserialize(body='{"test_missing_value_required_field": null}')
        self.assertEqual(
            u"valid value", self.portal.doc1.test_missing_value_required_field
        )
        self.assertEqual(
            (
                "test_missing_value_required_field is a required field.",
                "Setting it to null is not allowed.",
            ),
            cm.exception.args[0][0]["message"],
        )
        self.assertEqual(
            u"test_missing_value_required_field", cm.exception.args[0][0]["field"]
        )

    def test_set_layout(self):
        current_layout = self.portal.doc1.getLayout()
        self.assertNotEqual(current_layout, "my_new_layout")
        self.deserialize(body='{"layout": "my_new_layout"}')
        self.assertEqual("my_new_layout", self.portal.doc1.getLayout())


class TestDXContentSerializerDeserializer(unittest.TestCase):

    layer = PLONE_RESTAPI_DX_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]

        self.portal.invokeFactory(
            "DXTestDocument",
            id=u"doc1",
            test_textline_field=u"Test Document",
            test_readonly_field=u"readonly",
        )

        self.portal.invokeFactory(
            "DXTestDocument",
            id=u"doc2",
            test_textline_field=u"Test Document 2",
            test_readonly_field=u"readonly",
        )

    def deserialize(self, field, value, validate_all=False, context=None):
        context = context or self.portal.doc1
        body = {}
        body[field] = value
        body = json.dumps(body)
        self.request["BODY"] = body
        deserializer = getMultiAdapter((context, self.request), IDeserializeFromJson)
        return deserializer(validate_all=validate_all)

    def serialize(self, field):
        serializer = getMultiAdapter((self.portal.doc1, self.request), ISerializeToJson)
        return serializer()[field]

    def test_serialize2deserialize_relation(self):
        value = six.text_type(self.portal.doc2.UID())
        self.deserialize("test_relationchoice_field", value)

        serialization_value = self.serialize("test_relationchoice_field")

        self.deserialize("test_relationchoice_field", serialization_value)

        self.assertEqual(
            serialization_value["@id"],
            self.portal.doc1.test_relationchoice_field.to_object.absolute_url(),
        )
